package com.starbuds.server.service.task.job;

import cn.hutool.core.convert.Convert;
import com.starbuds.server.common.constant.activity.RankTypeEnum;
import com.starbuds.server.common.constant.cache.RedisCacheKeys;
import com.starbuds.server.common.pojo.activity.ActivityVO;
import com.starbuds.server.common.pojo.core.Tuple;
import com.starbuds.server.common.pojo.db.ActivityRankDO;
import com.starbuds.server.common.service.activity.ActivityService;
import com.starbuds.server.common.service.core.CacheService;
import com.starbuds.server.service.task.constant.JobHandlers;
import com.starbuds.server.service.task.dao.ActivityRankDAO;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.math.NumberUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;

/**
 * 活动排名记录持久化
 *
 * @author aoe
 * @date 2020/5/8
 */
@Component
@Slf4j
public class ActivityRankPersistenceJob {

    @Reference(version = "1.0.0")
    private CacheService cacheService;

    @Reference(version = "1.0.0")
    private ActivityService activityService;

    @Resource
    private ActivityRankDAO activityRankDAO;

    /**
     * 日志前缀
     */
    private String LOG_PREFIX = "活动排名记录久化处理 ";

    /**
     * （最后一次）活动排名记录久化处理
     *
     * @param param 活动类型（对应ActivityTypeEnum）
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @XxlJob(value = JobHandlers.ACTIVITY_RANK_LAST_RECORD)
    public ReturnT<String> lastRecordJobHandler(String param) {
        log.info(LOG_PREFIX + "活动类型id {}", param);
        if (StringUtils.isBlank(param)) {
            log.warn(LOG_PREFIX + "活动类型id {} 为空，不做记录", param);
            return ReturnT.SUCCESS;
        }
        if (!NumberUtils.isParsable(param)) {
            log.warn(LOG_PREFIX + "活动类型id {} 不是数字，不做记录", param);
            return ReturnT.SUCCESS;
        }

        Integer activityType = Convert.toInt(param);
        ActivityVO activity = activityService.getActivityByType(activityType);
        if (activity == null) {
            log.warn(LOG_PREFIX + "活动类型id {} 没有对应的活动，不做记录", param);
            return ReturnT.SUCCESS;
        }
        Long activityId = activity.getActivityId();

        for (RankTypeEnum rankTypeEnum : RankTypeEnum.values()) {
            String key = RedisCacheKeys.getActivityRankKey(activityId, rankTypeEnum.getValue());
            int insertCount = insertToDB(activityId, key, rankTypeEnum);
            if (insertCount > 0) {
                // 删除缓存
                Boolean delFlag = cacheService.delKey(key);
                log.info(LOG_PREFIX + "删除缓存 key={} delFlag={}", key, delFlag);
            } else {
                log.warn(LOG_PREFIX + "排行榜记录为空，不删除缓存 key={}", key);
            }
        }

        return ReturnT.SUCCESS;
    }

    /**
     * 保存排行榜记录到数据库
     *
     * @param activityId
     * @param key
     * @param rankTypeEnum
     */
    private int insertToDB(Long activityId, String key, RankTypeEnum rankTypeEnum) {
        Set<Tuple> tuples = cacheService.zRangeWithScores(key, 0, Long.MAX_VALUE);
        String logTag = "activityId=" + activityId + " key=" + key + " rankTypeEnum=" + rankTypeEnum;
        log.info(LOG_PREFIX + logTag + " Redis 总数={}", tuples.size());
        List<ActivityRankDO> rankDOList = new ArrayList<>(tuples.size());
        for (Tuple tuple : tuples) {
            ActivityRankDO rankDO = new ActivityRankDO();
            rankDO.setActivityId(activityId);
            rankDO.setRankType(rankTypeEnum.getValue());
            rankDO.setRankCreateTime(System.currentTimeMillis());

            String rankTargetId = tuple.getElement();
            if (StringUtils.isBlank(rankTargetId)) {
                log.warn(LOG_PREFIX + "参与活动的对象Id不存在，无法存储 {} {}", tuple, rankTypeEnum);
                continue;
            }
            if (!NumberUtils.isParsable(rankTargetId)) {
                log.warn(LOG_PREFIX + "参与活动的对象Id不是数字类型，无法存储 {} {}", tuple, rankTypeEnum);
                continue;
            }
            rankDO.setRankTargetId(Convert.toLong(rankTargetId));

            Double score = tuple.getScore();
            score = score == null ? 0 : score;
            rankDO.setScore(Convert.toLong(score));

            rankDOList.add(rankDO);
        }

        if (rankDOList.isEmpty()) {
            log.warn(LOG_PREFIX + logTag + " 排行榜记录为空，不执行保存记录");
            return 0;
        }

        int insertCount = activityRankDAO.batchInsert(rankDOList);
        log.info(LOG_PREFIX + logTag + " 保存记录={}", insertCount);
        return insertCount;
    }
}
