package com.starbuds.server.service.task.job;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.starbuds.server.common.constant.data.DataGreenTypeEnum;
import com.starbuds.server.common.constant.data.GreenAuditStatusEnum;
import com.starbuds.server.common.constant.data.GreenBizTypeEnum;
import com.starbuds.server.common.constant.feed.FeedStatusEnum;
import com.starbuds.server.common.constant.feed.FeedTypeEnum;
import com.starbuds.server.common.pojo.data.green.GreenAuditTrace;
import com.starbuds.server.common.pojo.db.DataGreenUserDO;
import com.starbuds.server.common.pojo.db.FeedDO;
import com.starbuds.server.common.pojo.db.FeedPhotoWallDO;
import com.starbuds.server.common.pojo.db.UserDO;
import com.starbuds.server.common.service.data.DataGreenService;
import com.starbuds.server.common.service.data.DataGreenUserService;
import com.starbuds.server.service.task.constant.JobHandlers;
import com.starbuds.server.service.task.dao.FeedPhotoWallDAO;
import com.starbuds.server.service.task.dao.UserDAO;
import com.starbuds.server.service.task.entity.user.UserVO;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;
import java.util.concurrent.*;

/***
 * @author qinjiang
 * @description 内容审核任务
 * @date 2021/9/2
 */
@Slf4j
@Component
public class CensorUserVerifyJob {

    //默认签名
    private final static String defaultUserSign = "you are my sunshine";
    //默认的头像地址
    private final static String defaultMaleAvator = "http://user.qn.panygo.com/avatar/default_male.png";
    private final static String defaultFemaleAvator = "http://user.qn.panygo.com/avatar/default_female.png";

    private final static String expireLayLibDomain = "http://user.sbqn.laylib.com";
    private final static String expirePaquDomain = "http://user.qn.paqukeji.com";

    @Autowired
    UserDAO userDAO;

    @Autowired
    FeedPhotoWallDAO feedPhotoWallDAO;

    @Reference(version = "1.0.0")
    private DataGreenUserService dataGreenUserService;

    @Reference(version = "1.0.0", timeout = 10000, retries = 0)
    private DataGreenService dataGreenService;

    /**
     * 审核用户信息(用户昵称、用户签名)
     * {"startTimestamp":121212,"endTimestamp":1223232}
     *
     * @param param
     * @return
     */
    @XxlJob(value = JobHandlers.CENSOR_USER_INFO_TASK)
    public ReturnT<String> censorUserInfoJobHandler(String param) {
        /*
         * 判断是否存在参数，存在参数按照参数进行执行
         * 没有参数则是全量进行执行，执行日期从2020年11月开始，月维度产生线程
         * */
        if (StringUtils.isNotBlank(param)) {
            boolean isJSONParam = validateJSONParam(param);
            if (!isJSONParam) {
                log.info("JobHandlers.CENSOR_PHOTO_WALL_TASK 参数不正确：" + param);
                return ReturnT.FAIL;
            }

            JSONObject jsonObject = JSON.parseObject(param);
            Long startTimestamp = jsonObject.getLong("startTimestamp");
            Long endTimestamp = jsonObject.getLong("endTimestamp");

            processSectionUserInfo(startTimestamp, endTimestamp);

            return ReturnT.SUCCESS;
        }

        long beginTimestamp = System.currentTimeMillis();
        //从2020年11月开始算，并以1个月为周期进行执行
        Calendar gatherCalendar = Calendar.getInstance();
        gatherCalendar.setTime(DateUtil.parse("202011", "yyyyMM"));

        Semaphore semaphore = new Semaphore(4);
        ThreadPoolExecutor threadPoolExecutor = new ThreadPoolExecutor(4, 6, 0, TimeUnit.MILLISECONDS, new LinkedBlockingDeque<>());

        boolean isRunloop = true;
        while (isRunloop) {
            Long startTimestamp = gatherCalendar.getTimeInMillis();
            //按月进行区分段
            gatherCalendar.add(Calendar.MONTH, 1);
            Long endTimestamp = gatherCalendar.getTimeInMillis();

            //终止现在的时间
            if (endTimestamp > System.currentTimeMillis()) {
                isRunloop = false;
            }

            threadPoolExecutor.execute(new Runnable() {
                @Override
                public void run() {
                    try {
                        semaphore.acquire();
                        processSectionUserInfo(startTimestamp, endTimestamp);
                        semaphore.release();
                    } catch (Exception e) {
                        semaphore.release();
                        e.printStackTrace();
                        log.error("--CENSOR_USER_INFO_TASK:TIME:startTime:" + startTimestamp + " endTime:" + endTimestamp);
                        log.error("--CENSOR_USER_INFO_TASK:ERROR:" + e.toString());
                    }
                }
            });
        }

        threadPoolExecutor.shutdown();
        while (true) {
            if (threadPoolExecutor.isTerminated()) {
                break;
            }
        }

        long endTimestamp = System.currentTimeMillis();
        System.out.println("\n\n\n\n\n完成用户扫描，共耗时：" + (endTimestamp - beginTimestamp) + "ms\n\n\n\n\n");

        return ReturnT.SUCCESS;
    }

    /**
     * 审核用户信息(用户头像)
     * {"startTimestamp":121212,"endTimestamp":1223232}
     *
     * @param param
     * @return
     */
    @XxlJob(value = JobHandlers.CENSOR_USER_AVATAR_TASK)
    public ReturnT<String> censorUserAvatorJobHander(String param) {
        /*
         * 判断是否存在参数，存在参数按照参数进行执行
         * 没有参数则是全量进行执行，执行日期从2020年11月开始，月维度产生线程
         * */
        if (StringUtils.isNotBlank(param)) {
            boolean isJSONParam = validateJSONParam(param);
            if (!isJSONParam) {
                log.info("JobHandlers.CENSOR_PHOTO_WALL_TASK 参数不正确：" + param);
                return ReturnT.FAIL;
            }

            JSONObject jsonObject = JSON.parseObject(param);
            Long startTimestamp = jsonObject.getLong("startTimestamp");
            Long endTimestamp = jsonObject.getLong("endTimestamp");

            processSectionUserAvatar(startTimestamp, endTimestamp);

            return ReturnT.SUCCESS;
        }

        long beginTimestamp = System.currentTimeMillis();
        //从2020年11月开始算，并以1个月为周期进行执行
        Calendar gatherCalendar = Calendar.getInstance();
        gatherCalendar.setTime(DateUtil.parse("202011", "yyyyMM"));

        Semaphore semaphore = new Semaphore(3);
        ThreadPoolExecutor threadPoolExecutor = new ThreadPoolExecutor(4, 6, 0, TimeUnit.MILLISECONDS, new LinkedBlockingDeque<>());

        boolean isRunloop = true;
        while (isRunloop) {
            Long startTimestamp = gatherCalendar.getTimeInMillis();
            //按月进行区分段
            gatherCalendar.add(Calendar.MONTH, 1);
            Long endTimestamp = gatherCalendar.getTimeInMillis();

            //终止现在的时间
            if (endTimestamp > System.currentTimeMillis()) {
                isRunloop = false;
            }

            threadPoolExecutor.execute(new Runnable() {
                @Override
                public void run() {
                    try {
                        semaphore.acquire();
                        processSectionUserAvatar(startTimestamp, endTimestamp);
                        semaphore.release();
                    } catch (Exception e) {
                        semaphore.release();
                        e.printStackTrace();
                        log.error("--CENSOR_USER_INFO_TASK:TIME:startTime:" + startTimestamp + " endTime:" + endTimestamp);
                        log.error("--CENSOR_USER_INFO_TASK:ERROR:" + e.toString());
                    }
                }
            });
        }

        threadPoolExecutor.shutdown();
        while (true) {
            if (threadPoolExecutor.isTerminated()) {
                break;
            }
        }

        long endTimestamp = System.currentTimeMillis();
        System.out.println("\n\n\n\n\n完成用户信息扫描，共耗时：" + (endTimestamp - beginTimestamp) + "ms\n\n\n\n\n");

        return ReturnT.SUCCESS;
    }

    /**
     * 审核用户照片墙
     *
     * @param param
     * @return
     */
    @XxlJob(value = JobHandlers.CENSOR_PHOTO_WALL_TASK)
    public ReturnT<String> censorPhotoWallJobHander(String param) {
        long begin = System.currentTimeMillis();
        boolean isJSONParam = validateJSONParam(param);
        if (!isJSONParam) {
            log.info("JobHandlers.CENSOR_PHOTO_WALL_TASK 参数不正确：" + param);
            return ReturnT.FAIL;
        }

        JSONObject jsonObject = JSON.parseObject(param);
        Long startTimestamp = jsonObject.getLong("startTimestamp");
        Long endTimestamp = jsonObject.getLong("endTimestamp");

        int pageIndex = 1;
        int pageSize = 1000;
        boolean isRunloop = true;
        while (isRunloop) {
            Semaphore semaphore = new Semaphore(8);
            ThreadPoolExecutor threadPoolExecutor = new ThreadPoolExecutor(8, 8, 0, TimeUnit.MILLISECONDS, new LinkedBlockingDeque<>());
            List<FeedPhotoWallDO> photoWallDOList = feedPhotoWallDAO.selectCensorPhotoWallList(startTimestamp, endTimestamp, (pageIndex - 1) * pageSize, pageSize);
            if (CollectionUtil.isEmpty(photoWallDOList) || photoWallDOList.size() <= 0) {
                break;
            }
            if (photoWallDOList.size() < pageSize) {
                isRunloop = false;
            }

            List<String> auditPhoneWallList = new ArrayList<>();
            for (FeedPhotoWallDO photoWallDo : photoWallDOList) {
                threadPoolExecutor.execute(new Runnable() {
                    @Override
                    public void run() {
                        try {
                            semaphore.acquire();
                            int passedRobot = GreenAuditStatusEnum.PASSED_ROBOT.value();
                            int approvedPhotoWallValue = auditImage(photoWallDo.getUserId(), photoWallDo.getUrl(), GreenBizTypeEnum.PHOTO_WALL).value();

                            if (approvedPhotoWallValue == GreenAuditStatusEnum.BLOCK_ROBOT.value()) {
                                //机审失败：直接逻辑删除
                                String updateSQL = String.format("UPDATE tb_feed_photo_wall SET `status`=%d,isDeleted=1 WHERE feedPhotoWallId = %d", FeedStatusEnum.REFUSE_ROBOT.value(), photoWallDo.getFeedPhotoWallId());
                                auditPhoneWallList.add(updateSQL);
                            } else if (approvedPhotoWallValue == GreenAuditStatusEnum.REVIEW_ROBOT.value()) {
                                //疑似状态不确定：置为待审核状态
                                String updateSQL = String.format("UPDATE tb_feed_photo_wall SET `status`=%d WHERE feedPhotoWallId = %d", FeedStatusEnum.REVIEW_ROBOT.value(), photoWallDo.getFeedPhotoWallId());
                                auditPhoneWallList.add(updateSQL);
                            }
                            semaphore.release();
                        } catch (InterruptedException e) {
                            semaphore.release();
                            e.printStackTrace();
                        }
                    }
                });
            }

            threadPoolExecutor.shutdown();
            while (true) {
                if (threadPoolExecutor.isTerminated()) {
                    break;
                }
            }
            if (auditPhoneWallList.size() > 0) {
                log.info("--JobHandlers.CENSOR_PHOTO_WALL_TASK:" + auditPhoneWallList.size());
                feedPhotoWallDAO.updateBatchCensorRecord(auditPhoneWallList);
            }
            pageIndex++;
        }

        long end = System.currentTimeMillis();
        log.info("--JobHandlers.CENSOR_PHOTO_WALL_TASK 执行用户照片墙完成，耗时：" + (end - begin) + "ms");
        return ReturnT.SUCCESS;
    }

    /**
     * 审核用户动态(动态图片、动态文字)
     *
     * @param param
     * @return
     */
    @XxlJob(value = JobHandlers.CENSOR_FEED_ACTIVITY_TASK)
    public ReturnT<String> censorFeedActivityJobHander(String param) {
        long begin = System.currentTimeMillis();
        boolean isJSONParam = validateJSONParam(param);
        if (!isJSONParam) {
            log.info("JobHandlers.CENSOR_FEED_ACTIVITY_TASK 参数不正确：" + param);
            return ReturnT.FAIL;
        }

        JSONObject jsonObject = JSON.parseObject(param);
        Long startTimestamp = jsonObject.getLong("startTimestamp");
        Long endTimestamp = jsonObject.getLong("endTimestamp");

        int pageIndex = 1;
        int pageSize = 1000;
        boolean isRunloop = true;

        while (isRunloop) {
            Semaphore semaphore = new Semaphore(8);
            ThreadPoolExecutor threadPoolExecutor = new ThreadPoolExecutor(8, 8, 0, TimeUnit.MILLISECONDS, new LinkedBlockingDeque<>());
            List<FeedDO> feedDOList = feedPhotoWallDAO.selectCensorFeedList(FeedTypeEnum.PHOTO.value(), startTimestamp, endTimestamp, (pageIndex - 1) * pageSize, pageSize);
            if (CollectionUtil.isEmpty(feedDOList) || feedDOList.size() <= 0) {
                break;
            }
            if (feedDOList.size() < pageSize) {
                isRunloop = false;
            }

            List<String> auditFeedDoIDList = new ArrayList<>();
            for (FeedDO feedDo : feedDOList) {
                threadPoolExecutor.execute(new Runnable() {
                    @Override
                    public void run() {
                        try {
                            semaphore.acquire();
                            CompletableFuture<GreenAuditStatusEnum> auditImageCompletableFuture = CompletableFuture.supplyAsync(() -> auditImage(feedDo.getUserId(), feedDo.getUrl(), GreenBizTypeEnum.FEED_IMAGE));
                            CompletableFuture<GreenAuditStatusEnum> auditTextCompletableFuture = CompletableFuture.supplyAsync(() -> auditText(feedDo.getUserId(), feedDo.getFeedTitle(), GreenBizTypeEnum.FEED_TEXT));

                            GreenAuditStatusEnum approvedImage = auditImageCompletableFuture.get();
                            GreenAuditStatusEnum approvedText = auditTextCompletableFuture.get();

                            int passedRobot = GreenAuditStatusEnum.PASSED_ROBOT.value();
                            if (approvedImage.value() != passedRobot || approvedText.value() != passedRobot) {
                                String updateSQL = String.format("UPDATE tb_feed SET feedStatus = %d,isDeleted = 1 WHERE feedId = %d", FeedStatusEnum.REFUSE_ROBOT.value(), feedDo.getFeedId());
                                auditFeedDoIDList.add(updateSQL);
                            }
                            semaphore.release();
                        } catch (Exception e) {
                            semaphore.release();
                            e.printStackTrace();
                        }
                    }
                });
            }

            threadPoolExecutor.shutdown();
            while (true) {
                if (threadPoolExecutor.isTerminated()) {
                    break;
                }
            }

            if (auditFeedDoIDList.size() > 0) {
                log.info("--JobHandlers.CENSOR_FEED_ACTIVITY_TASK:" + auditFeedDoIDList.size());
                feedPhotoWallDAO.updateBatchCensorRecord(auditFeedDoIDList);
            }

            pageIndex++;
        }

        long end = System.currentTimeMillis();
        log.info("--JobHandlers.CENSOR_FEED_ACTIVITY_TASK 审核用户动态执行完成，耗时：" + (end - begin) + "ms");

        return ReturnT.SUCCESS;
    }

    /**
     * 处理一段时间内的用户信息（昵称、签名）
     *
     * @param startTimestamp
     * @param endTimestamp
     */
    private void processSectionUserInfo(Long startTimestamp, Long endTimestamp) {
        long begin = System.currentTimeMillis();

        int pageIndex = 1;
        int pageSize = 2000;
        boolean isRunloop = true;
        while (isRunloop) {
            long beginPageTimestamp = System.currentTimeMillis();
            Semaphore semaphore = new Semaphore(16);
            ThreadPoolExecutor threadPoolExecutor = new ThreadPoolExecutor(16, 18, 0, TimeUnit.MILLISECONDS, new LinkedBlockingDeque<>());
            List<UserVO> users = userDAO.selectCensorUsersList(startTimestamp, endTimestamp, (pageIndex - 1) * pageSize, pageSize);
            if (CollectionUtil.isEmpty(users) || users.size() <= 0) {
                break;
            }
            if (users.size() < pageSize) {
                isRunloop = false;
            }

            List<UserDO> auditUserList = new ArrayList<>();
            for (UserVO userVO : users) {
                threadPoolExecutor.execute(new Runnable() {
                    @Override
                    public void run() {
                        try {
                            semaphore.acquire();
                            Long userID = userVO.getUserId();
                            String nickname = userVO.getUserName();
                            String usersign = userVO.getUserSign();

                            CompletableFuture<GreenAuditStatusEnum> auditUserNameCompletableFuture = CompletableFuture.supplyAsync(() -> auditText(userID, userVO.getUserName(), GreenBizTypeEnum.USER_NAME));
                            CompletableFuture<GreenAuditStatusEnum> auditUserSignCompletableFuture = CompletableFuture.supplyAsync(() -> auditText(userID, userVO.getUserSign(), GreenBizTypeEnum.USER_SIGN));

                            GreenAuditStatusEnum approvedUserName = auditUserNameCompletableFuture.get();
                            GreenAuditStatusEnum approvedUserSign = auditUserSignCompletableFuture.get();

                            boolean isUpdate = false;
                            int passedRobot = GreenAuditStatusEnum.PASSED_ROBOT.value();
                            if (approvedUserName.value() != passedRobot) {
                                nickname = String.format("用户%s", userVO.getUserNo());
                                isUpdate = true;
                            }
                            if (approvedUserSign.value() != passedRobot) {
                                usersign = null;
                                isUpdate = true;
                            }

                            if (isUpdate) {
                                UserDO user = new UserDO();
                                user.setUserId(userVO.getUserId());
                                user.setUserName(nickname);
                                user.setUserSign(usersign);
                                auditUserList.add(user);
                            }

                            semaphore.release();
                        } catch (Exception e) {
                            semaphore.release();
                            e.printStackTrace();
                        }
                    }
                });
            }

            threadPoolExecutor.shutdown();
            while (true) {
                if (threadPoolExecutor.isTerminated()) {
                    break;
                }
            }
            if (auditUserList.size() > 0) {
                userDAO.updateCensorUserInfoRecord(auditUserList);
            }

            long endPageTimestamp = System.currentTimeMillis();
            log.info("--JobHandlers.CENSOR_USER_INFO_TASK 用户信息(用户昵称、用户签名) :Page 当前页面处理用户:" + auditUserList.size() + "  耗时：" + (endPageTimestamp - beginPageTimestamp) + "ms");

            pageIndex++;
        }

        long end = System.currentTimeMillis();
        log.info("--JobHandlers.CENSOR_USER_INFO_TASK 执行用户信息(用户昵称、用户签名)完成,--section--startTime:" + startTimestamp + "  endTime:" + endTimestamp + ",  --总耗时(total)：" + (end - begin) + "ms");
    }

    /**
     * 处理一段时间内的用户信息(头像)
     *
     * @param startTimestamp
     * @param endTimestamp
     */
    private void processSectionUserAvatar(Long startTimestamp, Long endTimestamp) {
        long begin = System.currentTimeMillis();

        int pageIndex = 1;
        int pageSize = 1000;
        boolean isRunloop = true;
        while (isRunloop) {
            long beginPageTimestamp = System.currentTimeMillis();
            Semaphore semaphore = new Semaphore(6);
            ThreadPoolExecutor threadPoolExecutor = new ThreadPoolExecutor(6, 10, 0, TimeUnit.MILLISECONDS, new LinkedBlockingDeque<>());
            List<UserVO> users = userDAO.selectCensorUsersList(startTimestamp, endTimestamp, (pageIndex - 1) * pageSize, pageSize);
            if (CollectionUtil.isEmpty(users) || users.size() <= 0) {
                break;
            }
            if (users.size() < pageSize) {
                isRunloop = false;
            }

            List<UserDO> auditUserList = new ArrayList<>();
            for (UserVO userVO : users) {
                threadPoolExecutor.execute(new Runnable() {
                    @SneakyThrows
                    @Override
                    public void run() {
                        try {
                            semaphore.acquire();
                            Long userID = userVO.getUserId();
                            String useravator = userVO.getUserAvatar();
                            int passedRobot = GreenAuditStatusEnum.PASSED_ROBOT.value();

                            GreenAuditStatusEnum approvedAvator;
                            if (useravator == null) {
                                approvedAvator = GreenAuditStatusEnum.PASSED_ROBOT;
                            } else if (useravator.startsWith(expirePaquDomain) || useravator.startsWith(expireLayLibDomain)) {
                                approvedAvator = GreenAuditStatusEnum.BLOCK_ADMIN;
                            } else {
                                CompletableFuture<GreenAuditStatusEnum> auditAvatorCompletableFuture = CompletableFuture.supplyAsync(() -> auditImage(userID, userVO.getUserAvatar(), GreenBizTypeEnum.USER_AVATAR));
                                approvedAvator = auditAvatorCompletableFuture.get();
                            }

                            if (approvedAvator.value() != passedRobot) {
                                useravator = defaultMaleAvator;

                                if (userVO.getUserSex() == null) {
                                    useravator = defaultMaleAvator;
                                } else if (userVO.getUserSex().intValue() == 2) {
                                    useravator = defaultFemaleAvator;
                                }

                                UserDO user = new UserDO();
                                user.setUserId(userVO.getUserId());
                                user.setUserAvatar(useravator);
                                auditUserList.add(user);
                            }

                            semaphore.release();
                        } catch (Exception e) {
                            semaphore.release();
                            e.printStackTrace();
                        }
                    }
                });
            }

            threadPoolExecutor.shutdown();
            while (true) {
                if (threadPoolExecutor.isTerminated()) {
                    break;
                }
            }
            if (auditUserList.size() > 0) {
                userDAO.updateCensorUserAvatarRecord(auditUserList);
            }

            long endPageTimestamp = System.currentTimeMillis();
            log.info("--JobHandlers.CENSOR_USER_INFO_TASK 用户信息(用户头像) :Page 当前页面处理用户:" + auditUserList.size() + "  耗时：" + (endPageTimestamp - beginPageTimestamp) + "ms");

            pageIndex++;
        }

        long end = System.currentTimeMillis();
        log.info("--JobHandlers.CENSOR_USER_INFO_TASK 执行用户信息(用户头像)完成, --section--startTime:" + startTimestamp + " endTime:" + endTimestamp + "，--总耗时(total)：" + (end - begin) + "ms");
    }

    private boolean validateJSONParam(String param) {
        if (StringUtils.isBlank(param)) {
            return false;
        }
        JSONObject jsonObject = JSON.parseObject(param);
        if (jsonObject == null) {
            return false;
        }

        return jsonObject.keySet().contains("startTimestamp") && jsonObject.keySet().contains("endTimestamp");
    }

    /**
     * 审核文字
     *
     * @param userId
     * @param text
     * @param bizTypeEnum
     * @return
     */
    private GreenAuditStatusEnum auditText(Long userId, String text, GreenBizTypeEnum bizTypeEnum) {
        if (StringUtils.isBlank(text)) {
            return GreenAuditStatusEnum.PASSED_ROBOT;
        }

        if (text.equals(defaultUserSign)) {
            return GreenAuditStatusEnum.PASSED_ROBOT;
        }

        if (StringUtils.isNumeric(text)) {
            return GreenAuditStatusEnum.PASSED_ROBOT;
        }

        // 判断是否包含敏感词汇
        DataGreenUserDO dataGreenUser = new DataGreenUserDO();
        dataGreenUser.setUserId(userId);
        dataGreenUser.setData(text);
        dataGreenUser.setType(DataGreenTypeEnum.TEXT.getCode());
        dataGreenUser.setStatus(GreenAuditStatusEnum.AUDITING.value());
        dataGreenUser = dataGreenUserService.create(dataGreenUser);
        if (dataGreenUser == null) {
            return GreenAuditStatusEnum.PASSED_ROBOT;
        }

        String logBizHeader = bizTypeEnum.getValue();
        Long dataId = dataGreenUser.getDataId();
        log.info("{}审核开始 userId={} userName={} dataId={}", logBizHeader, userId, text, dataId);
        GreenAuditTrace greenAuditTrace = dataGreenService.textAudit(text, userId.toString(), dataId, bizTypeEnum);
        GreenAuditStatusEnum robotAuditStatus = greenAuditTrace.getRobotAuditStatus();
        log.info("{}审核接收数据 userId={} userName={} dataId={} robotAuditStatus={}", logBizHeader, userId, text, dataId, robotAuditStatus);

        DataGreenUserDO upPo = new DataGreenUserDO();
        upPo.setDataId(dataId);
        upPo.setStatus(robotAuditStatus.value());
        boolean update = dataGreenUserService.updateById(upPo);
        log.info("更新内容安全-用户信息 {} {}", update, upPo);

        return robotAuditStatus;
    }

    /**
     * 审核图片
     *
     * @param userId      用户ID
     * @param imgPath     图片路劲
     * @param bizTypeEnum 当前图片审核的业务类型
     * @return
     */
    private GreenAuditStatusEnum auditImage(Long userId, String imgPath, GreenBizTypeEnum bizTypeEnum) {
        if (StringUtils.isBlank(imgPath)) {
            return GreenAuditStatusEnum.PASSED_ROBOT;
        }
        if (defaultFemaleAvator.equals(imgPath) || defaultMaleAvator.equals(imgPath)) {
            return GreenAuditStatusEnum.PASSED_ROBOT;
        }

        DataGreenUserDO dataGreenUser = new DataGreenUserDO();
        dataGreenUser.setUserId(userId);
        dataGreenUser.setData(imgPath);
        dataGreenUser.setType(DataGreenTypeEnum.IMAGE.getCode());
        dataGreenUser.setStatus(GreenAuditStatusEnum.AUDITING.value());
        dataGreenUser = dataGreenUserService.create(dataGreenUser);
        if (dataGreenUser == null) {
            return GreenAuditStatusEnum.PASSED_ROBOT;
        }

        String logBizHeader = bizTypeEnum.getValue();
        Long dataId = dataGreenUser.getDataId();
        log.info("{}审核开始 userId={} url={} dataId={}", logBizHeader, userId, imgPath, dataId);
        GreenAuditTrace greenAuditTrace = dataGreenService.imgAudit(imgPath, userId.toString(), dataId, bizTypeEnum);
        GreenAuditStatusEnum robotAuditStatus = greenAuditTrace.getRobotAuditStatus();
        log.info("{}审核接收数据 userId={} url={} dataId={} robotAuditStatus={}", logBizHeader, userId, imgPath, dataId, robotAuditStatus);

        DataGreenUserDO upPo = new DataGreenUserDO();
        upPo.setDataId(dataId);
        upPo.setStatus(robotAuditStatus.value());
        boolean update = dataGreenUserService.updateById(upPo);
        log.info("更新内容安全-用户信息 {} {}", update, upPo);

        return robotAuditStatus;
    }
}
