package com.starbuds.server.service.task.job;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.util.ReUtil;
import cn.hutool.crypto.SecureUtil;
import com.starbuds.server.common.constant.cache.RedisCacheKeys;
import com.starbuds.server.common.pojo.core.Tuple;
import com.starbuds.server.common.pojo.db.*;
import com.starbuds.server.common.service.core.CacheService;
import com.starbuds.server.common.service.media.RankService;
import com.starbuds.server.service.task.constant.JobHandlers;
import com.starbuds.server.service.task.dao.*;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import com.xxl.job.core.util.ShardingUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * 排行榜持久化任务
 *
 * @author Lay
 * @date 2020/1/31
 */
@Component
@Slf4j
public class LiveRankPersistenceJob {

    @Reference(version = "1.0.0")
    CacheService cacheService;

    @Reference(version = "1.0.0")
    RankService rankService;

    @Resource
    LiveDailyDAO liveDailyDAO;

    @Resource
    LiveDailyRankDAO liveDailyRankDAO;

    @Resource
    LiveWeeklyDAO liveWeeklyDAO;

    @Resource
    LiveWeeklyRankDAO liveWeeklyRankDAO;

    @Resource
    LiveMonthlyDAO liveMonthlyDAO;

    @Resource
    LiveMonthlyRankDAO liveMonthlyRankDAO;

    @Resource
    LiveRankDAO liveRankDAO;

    /**
     * 直播日榜持久化处理
     *
     * @param param
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @XxlJob(value = JobHandlers.LIVE_RANK_DAILY)
    public ReturnT<String> dailyJobHandler(String param) {
        log.info("触发直播日榜持久化处理 {}", param);
        // 分片参数
        ShardingUtil.ShardingVO shardingVO = ShardingUtil.getShardingVo();
        // 查询日期处理
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
        Integer day;
        if (StringUtils.isNotBlank(param)) {
            // 有传参取传参的
            day = Convert.toInt(param);
        } else {
            // 不然就是取前一天的，因为定时任务一定是后一天来统计前一天的
            Calendar calendar = Calendar.getInstance();
            calendar.add(Calendar.DATE, -1);
            day = Convert.toInt(sdf.format(calendar.getTime()));
        }
        if (day != null) {
            // 从缓存中找出所有 key
            List<String> keys = cacheService.getKeys(RedisCacheKeys.getLiveRankDayPattern(day));
            int size = keys.size();

            List<String> cacheKeys = new ArrayList<>();
            for (int i = 0; i < size; i++) {
                if (shardingVO.getTotal() == 1 || i % shardingVO.getTotal() == shardingVO.getIndex()) {
                    // 命中分片，执行处理
                    cacheKeys.add(dailyPersistence(day, keys.get(i)));
                }
            }

            // 删除缓存
            cacheService.delKeys(cacheKeys);

            return ReturnT.SUCCESS;
        }
        return ReturnT.FAIL;
    }

    /**
     * 直播周榜持久化处理
     *
     * @param param
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @XxlJob(value = JobHandlers.LIVE_RANK_WEEKLY)
    public ReturnT<String> weeklyJobHander(String param) {
        log.info("直播周榜持久化处理 {}", param);
        // 分片参数
        ShardingUtil.ShardingVO shardingVO = ShardingUtil.getShardingVo();
        // 周数处理
        Integer weekNum;
        if (StringUtils.isNotBlank(param)) {
            weekNum = Convert.toInt(param);
        } else {
            // 取上一周
            String[] rankTimes = rankService.getRankTimes();
            weekNum = Convert.toInt(rankTimes[1]) - 1;
        }

        if (weekNum != null) {
            // 从缓存中找出所有 key
            List<String> keys = cacheService.getKeys(RedisCacheKeys.getLiveRankWeekPattern(weekNum));
            int size = keys.size();

            List<String> cacheKeys = new ArrayList<>();
            for (int i = 0; i < size; i++) {
                if (shardingVO.getTotal() == 1 || i % shardingVO.getTotal() == shardingVO.getIndex()) {
                    // 命中分片，执行处理
                    cacheKeys.add(weeklyPersistence(weekNum, keys.get(i)));
                }
            }

            // 删除缓存
            cacheService.delKeys(cacheKeys);

            return ReturnT.SUCCESS;
        }

        return ReturnT.FAIL;
    }

    /**
     * 直播月榜持久化处理
     *
     * @param param
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @XxlJob(value = JobHandlers.LIVE_RANK_MONTHLY)
    public ReturnT<String> monthlyJobHandler(String param) {
        log.info("直播月榜持久化处理 {}", param);
        // 分片参数
        ShardingUtil.ShardingVO shardingVO = ShardingUtil.getShardingVo();
        // 月份处理
        Integer monthNum;
        if (StringUtils.isNotBlank(param)) {
            monthNum = Convert.toInt(param);
        } else {
            // 取上个月
            SimpleDateFormat sdf = new SimpleDateFormat("yyyyMM");
            Calendar calendar = Calendar.getInstance();
            calendar.add(Calendar.MONTH, -1);
            monthNum = Convert.toInt(sdf.format(calendar.getTime()));
        }

        if (monthNum != null) {
            // 从缓存中找出所有 key
            List<String> keys = cacheService.getKeys(RedisCacheKeys.getLiveRankMonthPattern(monthNum));
            int size = keys.size();

            List<String> cacheKeys = new ArrayList<>();
            for (int i = 0; i < size; i++) {
                if (shardingVO.getTotal() == 1 || i % shardingVO.getTotal() == shardingVO.getIndex()) {
                    // 命中分片，执行处理
                    cacheKeys.add(monthlyPersistence(monthNum, keys.get(i)));
                }
            }

            // 删除缓存
            cacheService.delKeys(cacheKeys);

            return ReturnT.SUCCESS;
        }

        return ReturnT.FAIL;
    }

    /**
     * 日榜持久化
     *
     * @param day
     * @param key
     * @return
     */
    private String dailyPersistence(Integer day, String key) {
        Long anchorUserId = Convert.toLong(ReUtil.get(RedisCacheKeys.getLiveRankDayPattern(day).replace("*", "([0-9]+)"), key, 1));

        // 写入数据
        String dailyId = getDailyId(day, anchorUserId);
        LiveDailyDO liveDailyDO = new LiveDailyDO();
        liveDailyDO.setId(dailyId);
        liveDailyDO.setUserId(anchorUserId);
        liveDailyDO.setDateNum(day);

        // 写入排行榜数据
        Long liveRankAmountTotal = 0L;
        Set<Tuple> rankData = new TreeSet<>(((o1, o2) -> o2.getScore().compareTo(o1.getScore())));
        rankData.addAll(cacheService.zRangeWithScores(key, 0, -1));
        if (CollectionUtil.isNotEmpty(rankData)) {
            int rankNo = 0;
            List<LiveDailyRankDO> liveDailyRankDOList = new ArrayList<>();
            List<LiveRankDO> liveRankDOList = new ArrayList<>();
            for (Iterator<Tuple> iterator = rankData.iterator(); iterator.hasNext(); ) {
                rankNo++;

                Tuple tuple = iterator.next();
                Long userId = Convert.toLong(tuple.getElement());
                LiveDailyRankDO liveDailyRankDO = new LiveDailyRankDO();
                Long rankAmount = Convert.toLong(tuple.getScore());
                liveDailyRankDO.setId(getDailyRankId(day, anchorUserId, userId));
                liveDailyRankDO.setDailyId(dailyId);
                liveDailyRankDO.setRankNo(rankNo);
                liveDailyRankDO.setRankAmount(rankAmount);
                liveDailyRankDO.setUserId(userId);
                liveDailyRankDOList.add(liveDailyRankDO);

                Long curTime = System.currentTimeMillis();
                LiveRankDO liveRankDO = new LiveRankDO();
                liveRankDO.setAnchorUserId(anchorUserId);
                liveRankDO.setUserId(userId);
                liveRankDO.setRankValue(rankAmount);
                liveRankDO.setRankVersion(1);
                liveRankDO.setRankCreateTime(curTime);
                liveRankDO.setRankUpdateTime(curTime);
                liveRankDOList.add(liveRankDO);

                liveRankAmountTotal += rankAmount;
            }

            // 写入
            liveDailyDO.setLiveRankAmountTotal(liveRankAmountTotal);
            liveDailyDAO.insert(liveDailyDO);
            liveDailyRankDAO.batchInsert(liveDailyRankDOList);

            // 写入总榜
            liveRankDAO.batchInsertOrUpdate(liveRankDOList);
        }

        return key;
    }

    /**
     * 周榜持久化
     *
     * @param weekNum
     * @param key
     * @return
     */
    private String weeklyPersistence(Integer weekNum, String key) {
        Long anchorUserId = Convert.toLong(ReUtil.get(RedisCacheKeys.getLiveRankWeekPattern(weekNum).replace("*", "([0-9]+)"), key, 1));

        // 写入数据
        String id = getWeeklyId(weekNum, anchorUserId);
        LiveWeeklyDO liveWeeklyDO = new LiveWeeklyDO();
        liveWeeklyDO.setId(id);
        liveWeeklyDO.setUserId(anchorUserId);
        liveWeeklyDO.setWeekNum(weekNum);

        // 写入排行榜数据
        Long liveRankAmountTotal = 0L;
        Set<Tuple> rankData = new TreeSet<>(((o1, o2) -> o2.getScore().compareTo(o1.getScore())));
        rankData.addAll(cacheService.zRangeWithScores(key, 0, -1));
        if (CollectionUtil.isNotEmpty(rankData)) {
            int rankNo = 0;
            List<LiveWeeklyRankDO> rankDOList = new ArrayList<>();
            for (Iterator<Tuple> iterator = rankData.iterator(); iterator.hasNext(); ) {
                rankNo++;

                Tuple tuple = iterator.next();
                Long userId = Convert.toLong(tuple.getElement());
                LiveWeeklyRankDO rankDO = new LiveWeeklyRankDO();
                Long rankAmount = Convert.toLong(tuple.getScore());
                rankDO.setId(getWeeklyRankId(weekNum, anchorUserId, userId));
                rankDO.setWeeklyId(id);
                rankDO.setRankNo(rankNo);
                rankDO.setRankAmount(rankAmount);
                rankDO.setUserId(userId);
                rankDOList.add(rankDO);

                liveRankAmountTotal += rankAmount;
            }

            // 写入
            liveWeeklyDO.setLiveRankAmountTotal(liveRankAmountTotal);
            liveWeeklyDAO.insert(liveWeeklyDO);
            liveWeeklyRankDAO.batchInsert(rankDOList);
        }

        return key;
    }

    /**
     * 月榜持久化处理
     *
     * @param monthNum
     * @param key
     * @return
     */
    private String monthlyPersistence(Integer monthNum, String key) {
        Long anchorUserId = Convert.toLong(ReUtil.get(RedisCacheKeys.getLiveRankMonthPattern(monthNum).replace("*", "([0-9]+)"), key, 1));

        // 写入数据
        String id = getMonthlyId(monthNum, anchorUserId);
        LiveMonthlyDO liveMonthlyDO = new LiveMonthlyDO();
        liveMonthlyDO.setId(id);
        liveMonthlyDO.setUserId(anchorUserId);
        liveMonthlyDO.setMonthNum(monthNum);

        // 写入排行榜数据
        Long liveRankAmountTotal = 0L;
        Set<Tuple> rankData = new TreeSet<>(((o1, o2) -> o2.getScore().compareTo(o1.getScore())));
        rankData.addAll(cacheService.zRangeWithScores(key, 0, -1));
        if (CollectionUtil.isNotEmpty(rankData)) {
            int rankNo = 0;
            List<LiveMonthlyRankDO> rankDOList = new ArrayList<>();
            for (Iterator<Tuple> iterator = rankData.iterator(); iterator.hasNext(); ) {
                rankNo++;

                Tuple tuple = iterator.next();
                Long userId = Convert.toLong(tuple.getElement());
                LiveMonthlyRankDO rankDO = new LiveMonthlyRankDO();
                Long rankAmount = Convert.toLong(tuple.getScore());
                rankDO.setId(getMonthlyRankId(monthNum, anchorUserId, userId));
                rankDO.setMonthlyId(id);
                rankDO.setRankNo(rankNo);
                rankDO.setRankAmount(rankAmount);
                rankDO.setUserId(userId);
                rankDOList.add(rankDO);

                liveRankAmountTotal += rankAmount;
            }

            // 写入
            liveMonthlyDO.setLiveRankAmountTotal(liveRankAmountTotal);
            liveMonthlyDAO.insert(liveMonthlyDO);
            liveMonthlyRankDAO.batchInsert(rankDOList);
        }

        return key;
    }

    private static String getDailyId(Integer day, Long userId) {
        return SecureUtil.md5(String.format("%d:%d", day, userId));
    }

    private static String getDailyRankId(Integer day, Long anchorUserId, Long userId) {
        return SecureUtil.md5(String.format("%d:%d:%d", day, anchorUserId, userId));
    }

    private static String getWeeklyId(Integer weekNum, Long userId) {
        return SecureUtil.md5(String.format("%d:%d", weekNum, userId));
    }

    private static String getWeeklyRankId(Integer weekNum, Long anchorUserId, Long userId) {
        return SecureUtil.md5(String.format("%d:%d:%d", weekNum, anchorUserId, userId));
    }

    private static String getMonthlyId(Integer monthNum, Long userId) {
        return SecureUtil.md5(String.format("%d:%d", monthNum, userId));
    }

    private static String getMonthlyRankId(Integer monthNum, Long anchorUserId, Long userId) {
        return SecureUtil.md5(String.format("%d:%d:%d", monthNum, anchorUserId, userId));
    }
}
