package com.starbuds.server.service.task.job;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.util.NumberUtil;
import com.starbuds.server.common.constant.cache.RedisRtcKeys;
import com.starbuds.server.common.pojo.rtc.RtcRoomHeat;
import com.starbuds.server.common.service.data.RedisService;
import com.starbuds.server.common.service.rtc.RtcRoomService;
import com.starbuds.server.service.task.constant.JobHandlers;
import com.starbuds.server.service.task.dao.RtcRoomDAO;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * @Author jiangxiaowei
 * @DateTime 2020/6/24 11:26
 * @Description 语音房热度
 */
@Slf4j
@Component
public class RtcHeatJob {

    @Reference(version = "1.0.0")
    RedisService redisService;

    @Resource
    private RtcRoomDAO rtcRoomDAO;

    @Reference(version = "1.0.0", timeout = 5000)
    RtcRoomService rtcRoomService;

    @PostConstruct
    private void init() {
        // 初始化房间热度
        log.info("初始化房间热度 开始");
        rtcHeatHandler(null);
        log.info("初始化房间热度 结束");
    }

    @Transactional(rollbackFor = Exception.class)
    @XxlJob(value = JobHandlers.RTC_HEAT)
    public ReturnT<String> rtcHeatHandler(String param) {
        log.info("执行同步语音房热度开始 {}", param);

        List<Long> roomIds = getRoomIds();
        try {
            if (CollectionUtil.isEmpty(roomIds)) {
                log.warn("执行同步语音房热度结束 没有房间 {}", param);
                return ReturnT.SUCCESS;
            }

            updateHeat(roomIds);
            log.info("执行同步语音房热度结束 {}", param);
            return ReturnT.SUCCESS;
        } catch (Exception e) {
            log.error("执行同步语音房热度结束 失败", e);
            return ReturnT.FAIL;
        }
    }

    /**
     * 语音房id列表
     *
     * @return
     */
    private List<Long> getRoomIds() {
        return rtcRoomDAO.selectHeatRoomIds();
    }

    private void updateHeat(List<Long> roomIds) {
        List<RtcRoomHeat> heats = new ArrayList<>(roomIds.size());
        long curTime = System.currentTimeMillis();
        for (Long roomId : roomIds) {
            // 在线用户
            Long userValue = redisService.hashSize(RedisRtcKeys.getRtcRoomUser(roomId));
            userValue = userValue == null ? 0 : userValue * 100;

            Long giftValue = 0L;

            // 读取热度列表项
            String cachedKey = RedisRtcKeys.getRtcRoomHeatItem(roomId);
            Map<Object, Object> cachedMap = redisService.hashGetAll(cachedKey);
            if (CollectionUtil.isNotEmpty(cachedMap)) {
                List<String> keysToRemove = new ArrayList<>();
                // 判断是否过期
                for (Map.Entry<Object, Object> entry : cachedMap.entrySet()) {
                    long expireTime = Convert.toLong(entry.getKey(), 0L);
                    if (expireTime > curTime) {
                        // 未到期增加
                        giftValue += Convert.toLong(entry.getValue(), 0L);
                    } else {
                        // 到期移除
                        keysToRemove.add(entry.getKey().toString());
                    }
                }

                if (CollectionUtil.isNotEmpty(keysToRemove)) {
                    redisService.hashDel(cachedKey, keysToRemove.toArray(new String[0]));
                }
            }

            Long heatValue = userValue + giftValue;
            RtcRoomHeat roomHeat = new RtcRoomHeat();
            roomHeat.setRoomId(roomId);
            roomHeat.setHeatValue(heatValue);
            heats.add(roomHeat);
        }
        // 更新 DB
        rtcRoomDAO.batchUpdateHeat(heats);
        // 更新房间热度
        rtcRoomService.updateRoomHeat(heats);
    }

    /**
     * 更新热度
     * 1. 人数值 = 房间所有在线用户人数 * 100
     * 2. 礼物值 = 房间收到的所有礼物（有效期内）值的总和
     * 3. 房间热度 = 人数值 + 礼物值
     *
     * @param roomIds
     */
    private void updateHeatBak(List<Long> roomIds) {
        List<RtcRoomHeat> heats = new ArrayList<>(roomIds.size());
        for (Long roomId : roomIds) {
            // 在线用户
            Long userValue = redisService.hashSize(RedisRtcKeys.getRtcRoomUser(roomId));
            userValue = userValue == null ? 0 : userValue * 100;

            Long giftValue = 0L;
            // 在线用户数大于0时，才计算礼物的热度
            if (userValue > 0) {
                String giftKeysStr = RedisRtcKeys.getRtcRoomHeatDelayRoomId(roomId) + "*";
                List<String> giftKeys = redisService.getKeys(giftKeysStr);
                if (!CollectionUtils.isEmpty(giftKeys)) {
                    for (String giftKey : giftKeys) {
                        if (StringUtils.isBlank(giftKey)) {
                            continue;
                        }
                        String valueStr = giftKey.substring(giftKey.lastIndexOf("_") + 1);
                        if (!NumberUtil.isLong(valueStr)) {
                            log.error("房间礼物热度Key异常 {}", giftKey);
                            continue;
                        }
                        giftValue += Long.parseLong(valueStr);
                    }
                }
            }

            Long heatValue = userValue + giftValue;
            RtcRoomHeat roomHeat = new RtcRoomHeat();
            roomHeat.setRoomId(roomId);
            roomHeat.setHeatValue(heatValue);
            heats.add(roomHeat);
        }

        rtcRoomDAO.batchUpdateHeat(heats);
        rtcRoomService.updateRoomHeat(heats);
    }

}
