package com.starbuds.server.service.task.job;

import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.constant.message.sms.SmsAgentNameConstant;
import com.starbuds.server.common.constant.message.sms.SmsPushTaskSendStatusEnum;
import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.common.pojo.daoplus.QueryWrapper;
import com.starbuds.server.common.pojo.daoplus.UpdateWrapper;
import com.starbuds.server.common.pojo.db.SmsAgentDO;
import com.starbuds.server.common.pojo.message.SmsPushTaskDO;
import com.starbuds.server.common.pojo.message.bo.SendSmsUserBO;
import com.starbuds.server.common.pojo.message.qo.SmsPushTaskPageListQO;
import com.starbuds.server.common.pojo.message.qo.SmsPushTaskSendUsersPageListQO;
import com.starbuds.server.common.pojo.message.vo.SmsPushTaskVO;
import com.starbuds.server.common.pojo.util.PageListUtil;
import com.starbuds.server.common.service.message.SmsAgentService;
import com.starbuds.server.common.service.message.SmsProviderService;
import com.starbuds.server.common.service.message.SmsPushService;
import com.starbuds.server.common.service.message.SmsPushTaskService;
import com.starbuds.server.service.task.constant.JobHandlers;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 短信推送任务
 *
 * @author aoe
 * @date 2021/7/7
 */
@Slf4j
@Component
public class SmsPushTaskJob {

    @Reference(version = "1.0.0")
    private SmsPushTaskService smsPushTaskService;

    @Reference(version = "1.0.0")
    private SmsAgentService smsAgentService;

    @Reference(version = "1.0.0")
    private SmsProviderService smsProviderService;

    /**
     * PageList 中最大为 50
     */
    private static final Integer PAGE_SIZE = 50;

    @XxlJob(value = JobHandlers.SMS_PUSH_TASK)
    public ReturnT<String> smsPushTaskHandler(String param) {
        log.info("短信推送任务扫描开始 {}", param);

        SmsPushTaskPageListQO qo = new SmsPushTaskPageListQO();
        qo.setPageIndex(1);
        qo.setPageSize(PAGE_SIZE);
        qo.setSendStatus(SmsPushTaskSendStatusEnum.WAIT.getCode());
        qo.setIsDeleted(BoolIntVal.False);
        PageList<SmsPushTaskDO> pl = smsPushTaskService.getPageList(qo);
        PageList<SmsPushTaskVO> pageList = smsPushTaskService.toVoPageList(pl);
        List<SmsPushTaskVO> list = pageList.getList();
        if (CollectionUtils.isEmpty(list)) {
            log.info("短信推送任务扫描结束，没有要执行的任务 {}", param);
            return ReturnT.SUCCESS;
        }

        long now = System.currentTimeMillis();
        int sendCount = list.stream()
                .filter(task -> task.getTriggerTime() <= now)
                .mapToInt(this::sendSms)
                .sum();
        log.info("短信推送任务扫描结束，总共发送短信 {} 条", sendCount);
        return ReturnT.SUCCESS;
    }

    /**
     * 发送短信
     *
     * @param task
     * @return 发送成功的数量
     */
    private int sendSms(SmsPushTaskVO task) {
        // 更新短信推送任务状态：发送中
        QueryWrapper qw = new QueryWrapper();
        qw.eq("smsPushTaskId", task.getSmsPushTaskId());
        qw.eq("sendStatus", SmsPushTaskSendStatusEnum.WAIT.getCode());

        SmsPushTaskDO po = new SmsPushTaskDO();
        po.setSendTime(System.currentTimeMillis());
        po.setSendStatus(SmsPushTaskSendStatusEnum.SENDING.getCode());
        po.setUpdateTime(System.currentTimeMillis());

        UpdateWrapper uw = new UpdateWrapper();
        uw.setQw(qw);
        uw.setPo(po);
        boolean update = smsPushTaskService.update(uw);
        if (!update) {
            log.warn("短信推送任务-发送失败：更新任务状态为(发送中)失败 uw = " + JSON.toJSONString(uw));
            return 0;
        } else {
            log.info("短信推送任务：更新任务状态为(发送中) uw = {}", JSON.toJSONString(uw));
        }

        int sendCount = 0;
        SmsAgentDO smsAgent = smsAgentService.getSmsAgent(task.getAgentId());
        String failTask = " task = " + JSON.toJSONString(task);
        boolean sendFail = false;
        if (smsAgent == null) {
            log.error("短信推送任务发送失败：smsAgent is null agentId = " + task.getAgentId() + failTask);
            sendFail = true;
        }

        if (!SmsAgentNameConstant.BAIDU.equals(smsAgent.getAgentName()) && !SmsAgentNameConstant.YUEKE.equals(smsAgent.getAgentName())) {
            log.error("短信推送任务发送失败：短信服务商不在范围中！smsAgent = " + JSON.toJSONString(smsAgent) + failTask);
            sendFail = true;
        }

        String template = task.getTempContent();
        if (StringUtils.isBlank(template)) {
            log.error("短信推送任务发送失败：短信模板为空！smsAgent = " + JSON.toJSONString(smsAgent) + failTask);
            sendFail = true;
        }
        // 更新短信推送任务状态：发送失败
        if (sendFail) {
            qw = new QueryWrapper();
            qw.eq("smsPushTaskId", task.getSmsPushTaskId());
            qw.eq("sendStatus", SmsPushTaskSendStatusEnum.SENDING.getCode());

            po = new SmsPushTaskDO();
            po.setSendStatus(SmsPushTaskSendStatusEnum.FAIL.getCode());
            po.setActualSendCount(sendCount);
            po.setUpdateTime(System.currentTimeMillis());

            uw = new UpdateWrapper();
            uw.setQw(qw);
            uw.setPo(po);
            update = smsPushTaskService.update(uw);
            log.warn("短信推送任务发送失败：更新发送状态 = {} smsAgent = {}", update, JSON.toJSONString(smsAgent) + failTask);
            return sendCount;
        }

        synchronized (this) {
            // 发送短信相关配置
            String accessKey = smsAgent.getAgentAccessKey();
            String accessSecret = smsAgent.getAgentAccessSecret();
            String sign = task.getSignName();
            log.info("发送短信初始化参数 短信服务商={} {} accessKey={} accessSecret={} sign={}", smsAgent.getAgentName(), smsAgent.getAgentId(),
                    accessKey, accessSecret, sign);

            SmsPushTaskSendUsersPageListQO qo = new SmsPushTaskSendUsersPageListQO();
            qo.setLoginTimeStart(task.getLoginTimeStart());
            qo.setLoginTimeEnd(task.getLoginTimeEnd());
            qo.setRechargeStatus(smsPushTaskService.getRechargeStatus(task.getRechargeStatus()));
            qo.setPageSize(PAGE_SIZE);

            log.info("-- SmsPushTaskJob --:smsAgent:" + smsAgent);
            log.info("-- SmsPushTaskJob --:smsProviderService:" + smsProviderService);
            Boolean existInstance = smsProviderService.existInstance(smsAgent.getAgentName());
            if (!existInstance) {
                log.info("短信推送获取服务商为空...");
                return 0;
            }

            log.info("短信推送任务循环---------------------开始");
            final int initQps = 1;
            int maxQps = smsProviderService.maxQps(smsAgent.getAgentName());
            int qps = initQps;
            Integer pageIndex = 1;
            while (pageIndex != null) {
                qo.setPageIndex(pageIndex);
                PageList<SendSmsUserBO> pageList = smsPushTaskService.getSendSmsUserPageList(qo);
                log.info("短信推送任务循环中 pageIndex = {} total = {} list = {}", pageIndex, pageList.getTotal(),
                        JSON.toJSONString(pageList.getList(), true));
                pageIndex = PageListUtil.nextPage(pageList);
                log.info("短信推送任务循环中 nextPage = {}", pageIndex);

                List<SendSmsUserBO> list = pageList.getList();
                if (CollectionUtils.isEmpty(list)) {
                    break;
                }

                for (SendSmsUserBO sms : list) {
                    Map<String, String> varMap = new HashMap<>();
                    if (StringUtils.contains(template, "${content}")) {
                        varMap.put("content", sms.getUserName());
                    }
                    if (StringUtils.contains(template, "${userName}")) {
                        varMap.put("userName", sms.getUserName());
                    }
                    varMap.put("tempContent", template);
                    boolean sendFlag = smsProviderService.send(smsAgent.getAgentName(), sms.getUserPhone(), varMap, task.getTempCode(), accessKey, accessSecret, sign);
                    if (sendFlag) {
                        sendCount++;
                    }

                    if (++qps > maxQps) {
                        log.info("短信发送达到最大 QPS {} 等待 1 秒后继续", maxQps);
                        qps = initQps;
                        try {
                            wait(1000L);
                        } catch (InterruptedException e) {
                            log.error("短信发送达到最大 QPS " + maxQps + " 等待时被中断 pageList = " + JSON.toJSONString(pageList), e);
                            Thread.currentThread().interrupt();
                        }
                    }
                }
            }
            log.info("短信推送任务循环---------------------结束");
        }

        // 更新短信推送任务状态：发送成功
        qw = new QueryWrapper();
        qw.eq("smsPushTaskId", task.getSmsPushTaskId());
        qw.eq("sendStatus", SmsPushTaskSendStatusEnum.SENDING.getCode());

        po = new SmsPushTaskDO();
        po.setSendStatus(SmsPushTaskSendStatusEnum.SUCCESS.getCode());
        po.setActualSendCount(sendCount);
        po.setUpdateTime(System.currentTimeMillis());

        uw = new UpdateWrapper();
        uw.setQw(qw);
        uw.setPo(po);
        update = smsPushTaskService.update(uw);
        if (!update) {
            log.warn("短信推送任务：更新任务状态为(发送成功)失败 uw = " + JSON.toJSONString(uw));
        } else {
            log.info("短信推送任务：更新任务状态为(发送成功) uw = {}", JSON.toJSONString(uw));
        }

        return sendCount;
    }
}
