package com.starbuds.server.service.task.job;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.json.JSONUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.starbuds.server.common.constant.statistics.StatRtcRoomTypeEnum;
import com.starbuds.server.common.constant.statistics.StatTimeType;
import com.starbuds.server.common.pojo.db.StatRtcRoomDO;
import com.starbuds.server.common.pojo.rtc.RtcRoomGiftHis;
import com.starbuds.server.common.service.rtc.RtcRoomGiftHisService;
import com.starbuds.server.common.utils.DateUtil;
import com.starbuds.server.service.task.constant.JobHandlers;
import com.starbuds.server.service.task.dao.StatRtcRoomDAO;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.*;

/***
 * @author qinjiang
 * @description 语音房流水任务
 * @date 2021/8/24
 */
@Slf4j
@Component
public class StatRtcRoomStatementJob {
    private static final int pageSize = 50000;
    private static final SimpleDateFormat hourDateFormat = new SimpleDateFormat("yyyyMMddHH");
    private static final SimpleDateFormat dayDateFormat = new SimpleDateFormat("yyyyMMdd");
    private static final SimpleDateFormat monthDateFormat = new SimpleDateFormat("yyyyMM");

    @Reference(version = "1.0.0")
    RtcRoomGiftHisService rtcRoomGiftHisService;

    @Autowired
    StatRtcRoomDAO statRtcRoomDAO;

    /**
     * 统计房间赠送礼物流水
     * {"localTimestamp":1631596049490}
     *
     * @param param
     * @return
     */
    @XxlJob(value = JobHandlers.STAT_RTCROOM_STATEMENT_TASK)
    public ReturnT<String> statRtcRoomStatement(String param) {
        /**
         * 1、统计昨天每小时的数据；
         * 2、插入小时数据、并更新日、周、月、年数据
         * 3、清除昨日数据
         *
         * 从 tb_rtc_room_gift_his 中查找昨日数据，统计数据记录处于某个小时时段内，插入 tb_stat_rtc_room 表中，并连带统计当月
         */

        Long begin = System.currentTimeMillis();

        long actionTimestamp = System.currentTimeMillis();
        if (StringUtils.isNotBlank(param)) {
            boolean isJSONParam = JSONUtil.isJson(param);
            if (!isJSONParam) {
                log.info("JobHandlers.STAT_RTCROOM_STATEMENT_TASK 参数不正确：" + param);
                return ReturnT.FAIL;
            }

            JSONObject jsonObject = JSON.parseObject(param);
            if (jsonObject.containsKey("localTimestamp")) {
                actionTimestamp = jsonObject.getLong("localTimestamp");
            }
        }

        Map<String, StatRtcRoomDO> targetDayMap = new HashMap<>(5);
        Map<String, StatRtcRoomDO> targetHourMap = new HashMap<>(24);
        Long yesterDayTimestamp = DateUtil.getTimestamp(actionTimestamp, 2L) - 24 * 60 * 60 * 1000;
        Long statTimeDay = Convert.toLong(dayDateFormat.format(yesterDayTimestamp));

        int pageIndex = 1;
        boolean isRunloop = true;
        while (isRunloop) {
            List<RtcRoomGiftHis> giftHisList = rtcRoomGiftHisService.getRtcRoomGiftHisRecord(yesterDayTimestamp, (pageIndex - 1) * pageSize, pageSize);
            if (CollectionUtil.isEmpty(giftHisList) || giftHisList.size() == 0) {
                break;
            }
            if (giftHisList.size() < pageSize) {
                isRunloop = false;
            }

            Iterator<RtcRoomGiftHis> iterator = giftHisList.iterator();
            while (iterator.hasNext()) {
                RtcRoomGiftHis giftHis = iterator.next();

                boolean isFamily = giftHis.getRoomFamilyId().longValue() > 0;
                StatRtcRoomTypeEnum roomTypeEnum = transStatType(isFamily, StatRtcRoomTypeEnum.toEnum(giftHis.getGiftType().intValue()));
                Date createDate = new Date(giftHis.getCreatetime());
                Long statTimeHour = Convert.toLong(hourDateFormat.format(createDate));

                //小时维度记录
                String hisHourKey = String.format("%d_%d_%d", giftHis.getRoomId(), roomTypeEnum.getValue(), statTimeHour);
                StatRtcRoomDO statRtcRoomDO = targetHourMap.get(hisHourKey);
                if (statRtcRoomDO == null) {
                    statRtcRoomDO = new StatRtcRoomDO();
                    statRtcRoomDO.setRoomId(giftHis.getRoomId());
                    statRtcRoomDO.setStatTimeType(StatTimeType.YEAR_MONTH_DAY_HOUR);
                    statRtcRoomDO.setStatTime(statTimeHour);
                    statRtcRoomDO.setStatType(roomTypeEnum.getValue());
                    statRtcRoomDO.setStatValue(new BigDecimal(0));
                }

                BigDecimal statHourValue = statRtcRoomDO.getStatValue().add(new BigDecimal(giftHis.getPrice()));
                statRtcRoomDO.setStatValue(statHourValue);
                targetHourMap.put(hisHourKey, statRtcRoomDO);

                //天维度记录(类型不一样)
                String hisDayKey = String.format("%d_%d", giftHis.getRoomId(), roomTypeEnum.getValue());
                StatRtcRoomDO statDayRtcRoomDO = targetDayMap.get(hisDayKey);
                if (statDayRtcRoomDO == null) {
                    statDayRtcRoomDO = new StatRtcRoomDO();
                    statDayRtcRoomDO.setRoomId(giftHis.getRoomId());
                    statDayRtcRoomDO.setStatTimeType(StatTimeType.YEAR_MONTH_DAY);
                    statDayRtcRoomDO.setStatTime(statTimeDay);
                    statDayRtcRoomDO.setStatType(roomTypeEnum.getValue());
                    statDayRtcRoomDO.setStatValue(new BigDecimal(0));
                }

                BigDecimal statDayValue = statDayRtcRoomDO.getStatValue().add(new BigDecimal(giftHis.getPrice()));
                statDayRtcRoomDO.setStatValue(statDayValue);
                targetDayMap.put(hisDayKey, statDayRtcRoomDO);
            }

            pageIndex++;
        }

        List<String> batchSQL = new ArrayList<>();
        List<StatRtcRoomDO> targetDOList = new ArrayList<>();
        targetDOList.addAll(targetHourMap.values());
        targetDOList.addAll(targetDayMap.values());
        targetDOList.forEach(statRtcRoomDO -> {
            String sqlFormat = "INSERT INTO tb_stat_rtc_room_gift(statTime,statTimeType,roomId,statType,statValue) VALUES (%d,%d,%d,%d,%d) ON DUPLICATE KEY UPDATE statValue = %d";
            String updateSQL = String.format(sqlFormat, statRtcRoomDO.getStatTime(), statRtcRoomDO.getStatTimeType(), statRtcRoomDO.getRoomId(),
                    statRtcRoomDO.getStatType(), statRtcRoomDO.getStatValue().intValue(), statRtcRoomDO.getStatValue().intValue());
            batchSQL.add(updateSQL);
        });

        if (batchSQL.size() > 0) {
            statRtcRoomDAO.insertOrUpdateBatch(batchSQL);
        }

        //统计顺序 -->  3：月  ---> 4：年
        statCycleValue(yesterDayTimestamp, 3);
        statCycleValue(yesterDayTimestamp, 4);

        Long end = System.currentTimeMillis();
        log.info("[statRtcRoomStatement]数据执行共耗时 :" + (end - begin) + "毫秒");

        return ReturnT.SUCCESS;
    }

    /**
     * 判断当前标记类型是普通流水还是家族流水
     *
     * @param isFamily
     * @param statEnum
     * @return
     */
    private StatRtcRoomTypeEnum transStatType(boolean isFamily, StatRtcRoomTypeEnum statEnum) {
        StatRtcRoomTypeEnum statRtcRoomTypeEnum = statEnum;

        if (isFamily) {
            switch (statEnum) {
                case NORMAL_GIFT_DIAMOND:
                    statRtcRoomTypeEnum = StatRtcRoomTypeEnum.NORMAL_GIFT_DIAMOND_FAMILY_USER;
                    break;
                case LOTTERY_GIFT_DIAMOND:
                    statRtcRoomTypeEnum = StatRtcRoomTypeEnum.LOTTERY_GIFT_DIAMOND_FAMILY_USER;
                    break;
                case BAG_GIFT_DIAMOND:
                    statRtcRoomTypeEnum = StatRtcRoomTypeEnum.BAG_GIFT_DIAMOND_FAMILY_USER;
                    break;
            }
        }

        return statEnum;
    }

    /**
     * 统计周期值
     *
     * @param timestamp 统计的时间
     * @param scope     统计的时间类型 4:年 3：月 2：日
     */
    private void statCycleValue(long timestamp, int scope) {
        if (scope < 2 && scope > 4) {
            return;
        }
        //统计的时间类型，存储数据用
        int statTimeType = 0;
        //获取格式化时间的类型:年->月；月->日；日->时
        SimpleDateFormat targetDateFormat = null;
        //定义获取时间间隔日历
        Calendar gatherCalendar = Calendar.getInstance();
        gatherCalendar.setTimeInMillis(timestamp);
        if (scope == 4) {
            gatherCalendar.add(Calendar.YEAR, 1);
            targetDateFormat = monthDateFormat;
            statTimeType = StatTimeType.YEAR;
        } else if (scope == 3) {
            gatherCalendar.add(Calendar.MONTH, 1);
            targetDateFormat = dayDateFormat;
            statTimeType = StatTimeType.YEAR_MONTH;
        } else if (scope == 2) {
            gatherCalendar.add(Calendar.DAY_OF_MONTH, 1);
            targetDateFormat = hourDateFormat;
            statTimeType = StatTimeType.YEAR_MONTH_DAY;
        }

        //当前时间戳
        Long beginTimestamp = DateUtil.getTimestamp(timestamp, (long) scope);
        Long endTimestamp = DateUtil.getTimestamp(gatherCalendar.getTimeInMillis(), (long) scope);

        //本月 与 下个月  的月初时间
        Long beginScope = Convert.toLong(targetDateFormat.format(beginTimestamp));
        Long endScope = Convert.toLong(targetDateFormat.format(endTimestamp));

        Map<String, StatRtcRoomDO> targetMap = new HashMap<>();

        int pageIndex = 1;
        boolean isRunloop = true;

        while (isRunloop) {
            List<StatRtcRoomDO> giftHisList = rtcRoomGiftHisService.getRtcRoomGiftBySocpe(beginScope, endScope, (pageIndex - 1) * pageSize, pageSize);
            if (CollectionUtil.isEmpty(giftHisList) || giftHisList.size() <= 0) {
                break;
            }
            if (giftHisList.size() < pageSize) {
                isRunloop = false;
            }

            Iterator<StatRtcRoomDO> iterator = giftHisList.iterator();
            while (iterator.hasNext()) {
                StatRtcRoomDO giftHis = iterator.next();
                //2021090910  每除以100后将时段会进一个维度；如：202109 月维度 / 100  =  年维度
                Long statTime = giftHis.getStatTime().longValue() / 100;

                String hisKey = String.format("%d_%d_%d", giftHis.getRoomId(), giftHis.getStatType(), statTime);
                StatRtcRoomDO statRtcRoomDO = targetMap.get(hisKey);
                if (statRtcRoomDO == null) {
                    statRtcRoomDO = new StatRtcRoomDO();
                    statRtcRoomDO.setRoomId(giftHis.getRoomId());
                    statRtcRoomDO.setStatTimeType(statTimeType);
                    statRtcRoomDO.setStatTime(statTime);
                    statRtcRoomDO.setStatType(giftHis.getStatType());
                    statRtcRoomDO.setStatValue(new BigDecimal(0));
                }

                BigDecimal statValue = statRtcRoomDO.getStatValue().add(giftHis.getStatValue());
                statRtcRoomDO.setStatValue(statValue);
                targetMap.put(hisKey, statRtcRoomDO);
            }

            pageIndex++;
        }

        List<String> batchSQL = new ArrayList<>();
        Collection<StatRtcRoomDO> statValues = targetMap.values();
        statValues.forEach(statRtcRoomDO -> {
            String sqlFormat = "INSERT INTO tb_stat_rtc_room_gift(statTime,statTimeType,roomId,statType,statValue) VALUES (%d,%d,%d,%d,%d) ON DUPLICATE KEY UPDATE statValue = %d";
            String updateSQL = String.format(sqlFormat, statRtcRoomDO.getStatTime(), statRtcRoomDO.getStatTimeType(), statRtcRoomDO.getRoomId(),
                    statRtcRoomDO.getStatType(), statRtcRoomDO.getStatValue().intValue(), statRtcRoomDO.getStatValue().intValue());
            batchSQL.add(updateSQL);
        });

        if (batchSQL.size() > 0) {
            statRtcRoomDAO.insertOrUpdateBatch(batchSQL);
        }
    }

}
