package com.starbuds.server.service.task.job;

import cn.hutool.core.convert.Convert;
import com.starbuds.server.common.constant.finance.PayStatusEnum;
import com.starbuds.server.common.constant.statistics.StatUserTypeEnum;
import com.starbuds.server.common.pojo.db.UserInviteDO;
import com.starbuds.server.service.task.constant.JobHandlers;
import com.starbuds.server.service.task.dao.PayOrderDAO;
import com.starbuds.server.service.task.dao.UserDAO;
import com.starbuds.server.service.task.dao.UserInviteDAO;
import com.starbuds.server.service.task.entity.UserRechargeDTO;
import com.starbuds.server.service.task.provider.StatUserProvider;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Author jiangxiaowei
 * @DateTime 2020/6/24 11:26
 * @Description 用户统计
 */
@Slf4j
@Component
public class StatUserJob {

    @Resource
    UserInviteDAO userInviteDAO;

    @Resource
    UserDAO userDAO;

    @Resource
    PayOrderDAO payOrderDAO;

    @Resource
    StatUserProvider statUserProvider;

    @Transactional(rollbackFor = Exception.class)
    @XxlJob(value = JobHandlers.STAT_USER)
    public ReturnT<String> statUserHandler(String param) {
        log.info("执行用户统计 {}", param);

        long curTime = System.currentTimeMillis();

        try {
            if (StringUtils.isEmpty(param)) {
                log.info("没有入参，不处理");
                return ReturnT.SUCCESS;
            }
            String[] strArr = param.split(",");
            if (strArr.length != 2) {
                log.info("参数不正确，不处理");
                return ReturnT.SUCCESS;
            }
            // 处理类型
            String typeParam = strArr[0];
            String timeParam = strArr[1];
            // 查询这个时间以前的数据
            Long beforeTime = null;
            // 第一个参数
            if (!"1,2".contains(typeParam)) {
                log.info("参数类型不正确，不处理");
                return ReturnT.SUCCESS;
            }
            // 第二个参数
            if ("0".equals(timeParam)) {
                beforeTime = curTime;
            } else if ("-1".equals(timeParam)) {
                // 2020-06-24 20:30:00
                Long defaultTime = 1593001800000L;
                beforeTime = defaultTime;
            } else {
                beforeTime = Convert.toLong(timeParam);
            }
            log.info("查询这个时间以前的数据，{}", beforeTime);

            // 分页大小
            int pageSize = 5000;

            if ("1".equals(typeParam)) {
                log.info("处理充值数据");
                // 总大小
                int payTotalNum = payOrderDAO.countUserRecharge(PayStatusEnum.SUCCESSED.getValue(), beforeTime);
                if (payTotalNum == 0) {
                    log.info("无充值数据");
                    return ReturnT.SUCCESS;
                }
                // 分页数量
                int pageNum = payTotalNum / pageSize;
                if (payTotalNum % pageSize > 0) {
                    pageNum += 1;
                }
                for (int i = 0; i < pageNum; i++) {
                    int pageCursor = i * pageSize;
                    log.info("正在处理充值数据，共需处理{}条，共{}页,当前处理第{}页", payTotalNum, pageNum, i + 1);
                    // 充值数据
                    List<UserRechargeDTO> userRecharges = payOrderDAO.selectUserRecharge(PayStatusEnum.SUCCESSED.getValue(), beforeTime, pageCursor, pageSize);
                    if (CollectionUtils.isEmpty(userRecharges)) {
                        log.info("无充值数据");
                        continue;
                    }
                    // 充值金额和充值次数
                    for (UserRechargeDTO dto : userRecharges) {
                        statUserProvider.addStatAsync(dto.getUserId(), StatUserTypeEnum.RECHARGE_AMOUNT.getValue(), dto.getOrderAmount(), dto.getPayOrderCreateTime());
                        statUserProvider.addStatAsync(dto.getUserId(), StatUserTypeEnum.RECHARGE_TIMES.getValue(), BigDecimal.ONE, dto.getPayOrderCreateTime());
                    }
                    log.info("第{}页,充值数据已统计完成，共需处理{}条，分{}页", i + 1, payTotalNum, pageNum);
                }
                log.info("充值数据已统计完成");
            } else if ("2".equals(typeParam)) {
                log.info("处理邀请数据");
                // 总大小
                int inviteTotalNum = userInviteDAO.countInvites(beforeTime);
                if (inviteTotalNum == 0) {
                    log.info("无邀请数据");
                    return ReturnT.SUCCESS;
                }
                // 分页数量
                int pageNum = inviteTotalNum / pageSize;
                if (inviteTotalNum % pageSize > 0) {
                    pageNum += 1;
                }

                // 充值数据
                List<UserRechargeDTO> recharges = userDAO.selectUserRecharge(beforeTime, PayStatusEnum.SUCCESSED.getValue());
                // 用户id=>充值记录
                Map<Long, List<UserRechargeDTO>> rechargeMap = recharges.stream().collect(Collectors.groupingBy(UserRechargeDTO::getUserId));

                // 用户id=>首充时间
                Map<Long, Long> userFirstRechargeTimeMap = new HashMap<>();

                for (Map.Entry<Long, List<UserRechargeDTO>> map : rechargeMap.entrySet()) {
                    List<UserRechargeDTO> value = map.getValue();
                    long firstRechargeTime = value.stream().mapToLong(UserRechargeDTO::getPayOrderCreateTime).min().getAsLong();
                    userFirstRechargeTimeMap.put(map.getKey(), firstRechargeTime);
                }

                for (int i = 0; i < pageNum; i++) {
                    int pageCursor = i * pageSize;
                    log.info("正在处理邀请数据，共需处理{}条，分{}页,当前处理第{}页", inviteTotalNum, pageNum, i + 1);
                    // 邀请数据
                    List<UserInviteDO> inviteList = userInviteDAO.selectInvites(beforeTime, pageCursor, pageSize);
                    if (CollectionUtils.isEmpty(inviteList)) {
                        log.info("无邀请数据");
                        continue;
                    }

                    for (UserInviteDO dto : inviteList) {
                        // 被邀请人
                        Long userId = dto.getUserId();
                        // 邀请人
                        Long inviterUserId = dto.getInviterUserId();
                        // 邀请用户数
                        statUserProvider.addStatAsync(inviterUserId, StatUserTypeEnum.INVITE_NUM.getValue(), BigDecimal.ONE, dto.getInviteTime());

                        // 邀请充值用户数 和 充值金额
                        if (rechargeMap.keySet().contains(userId)) {
                            statUserProvider.addStatAsync(inviterUserId, StatUserTypeEnum.INVITE_RECHARGE_NUM.getValue(), BigDecimal.ONE, userFirstRechargeTimeMap.get(userId));
                            log.info("-邀请人:{},被邀请人id:{},邀请充值用户数已统计,首充时间:{}", inviterUserId, userId, userFirstRechargeTimeMap.get(userId));

                            // 邀请用户的充值记录
                            List<UserRechargeDTO> userRecharges = rechargeMap.get(userId);
                            if (!CollectionUtils.isEmpty(userRecharges)) {
                                for (UserRechargeDTO rechargeDTO : userRecharges) {
                                    statUserProvider.addStatAsync(inviterUserId, StatUserTypeEnum.INVITE_RECHARGE_AMOUNT.getValue(), rechargeDTO.getOrderAmount(), rechargeDTO.getPayOrderCreateTime());
                                    log.info("--邀请人id:{},被邀请人id:{},邀请用户充值金额已统计,邀请用户充值金额:{},充值时间:{}", inviterUserId, userId, rechargeDTO.getOrderAmount(), rechargeDTO.getPayOrderCreateTime());
                                }
                            }
                        }
                    }
                    log.info("第{}页，邀请数据已处理，共需处理{}条，分{}页", i + 1, inviteTotalNum, pageNum);
                }
                log.info("充值数据已统计完成");
            }
        } catch (Exception e) {
            log.info("执行出错", e);
            return ReturnT.FAIL;
        }

        long endTime = System.currentTimeMillis();

        log.info("执行成功,共耗时{}s", (endTime - curTime) / 1000);
        return ReturnT.SUCCESS;
    }

}
