package com.starbuds.server.service.task.provider;

import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.EasyExcelFactory;
import com.alibaba.excel.ExcelWriter;
import com.alibaba.excel.write.metadata.WriteSheet;
import com.alibaba.excel.write.metadata.style.WriteCellStyle;
import com.alibaba.excel.write.metadata.style.WriteFont;
import com.alibaba.excel.write.style.HorizontalCellStyleStrategy;
import com.google.common.base.Preconditions;
import com.starbuds.server.service.task.entity.excel.FreezeAndFilter;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.HorizontalAlignment;
import org.apache.poi.ss.usermodel.IndexedColors;
import org.apache.poi.ss.usermodel.VerticalAlignment;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.io.File;
import java.time.LocalDate;
import java.util.List;

/**
 * Excel工具类
 * EasyExcel https://github.com/alibaba/easyexcel
 *
 * @author aoe
 * @date 2021/8/25
 */
@Slf4j
@Component
public class ExcelProvider {

    @Value("${excel.path}")
    public String path;

    /**
     * 保存Excel文件到磁盘
     *
     * @param fileName  Excel 文件名称
     * @param sheetName sheet 名称
     * @param list      导出数据
     * @param clazz     EasyExcel 对应的类
     */
    public void save2Disk(String fileName, String sheetName, List list, Class clazz) {
        Preconditions.checkArgument(StringUtils.isNotBlank(fileName), "保存Excel文件 fileName 不能为 null");
        Preconditions.checkArgument(fileName.endsWith("xlsx"), "Excel文件后缀必须为 xlsx ！");
        Preconditions.checkArgument(StringUtils.isNotBlank(sheetName), "保存Excel文件 sheetName 不能为 null");
        Preconditions.checkNotNull(list, "保存Excel文件 list 不能为 null");
        log.info("保存Excel文件 fileName={} sheets size = {}", fileName, list.size());

        ExcelWriter excelWriter = EasyExcel.write(getDir() + fileName, clazz).build();
        WriteSheet writeSheet = EasyExcelFactory.writerSheet(0, sheetName)
                .head(clazz)
                .registerWriteHandler(new FreezeAndFilter())
                .registerWriteHandler(getHorizontalCellStyleStrategy())
                .build();
        excelWriter.write(list, writeSheet);
        excelWriter.finish();
        log.info("保存Excel文件 {} 共 {} 条数据", fileName, list.size());
    }

    public String getDir() {
        LocalDate now = LocalDate.now();
        String monthDir = String.format("%s-%s/", now.getYear(), now.getMonthValue());
        String diskPath = path + monthDir;
        File dir = new File(diskPath);
        if (!dir.exists()) {
            dir.mkdirs();
        }
        return diskPath;
    }

    /**
     * 单元格策略
     *
     * @return
     */
    private static HorizontalCellStyleStrategy getHorizontalCellStyleStrategy() {
        // 标题策略
        WriteCellStyle headWriteCellStyle = new WriteCellStyle();
        // 背景色
        headWriteCellStyle.setFillForegroundColor(IndexedColors.PALE_BLUE.getIndex());

        WriteFont headWriteFont = new WriteFont();
        headWriteFont.setFontHeightInPoints((short) 12);
        headWriteFont.setFontName("宋体");

        headWriteCellStyle.setWriteFont(headWriteFont);

        // 内容的策略
        WriteCellStyle contentWriteCellStyle = new WriteCellStyle();

        WriteFont contentWriteFont = new WriteFont();
        // 字体大小
        contentWriteFont.setFontHeightInPoints((short) 12);
        contentWriteFont.setFontName("宋体");

        contentWriteCellStyle.setWriteFont(contentWriteFont);
        // 设置 自动换行
        contentWriteCellStyle.setWrapped(true);
        // 设置 垂直居中
        contentWriteCellStyle.setVerticalAlignment(VerticalAlignment.CENTER);
        // 设置 水平居中
        contentWriteCellStyle.setHorizontalAlignment(HorizontalAlignment.CENTER);

        // 这个策略是 头是头的样式 内容是内容的样式 其他的策略可以自己实现
        return new HorizontalCellStyleStrategy(headWriteCellStyle, contentWriteCellStyle);
    }
}
