package com.starbuds.server.service.task.provider;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.convert.Convert;
import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.constant.data.TaskStatusEnum;
import com.starbuds.server.common.constant.data.TaskTypeEnum;
import com.starbuds.server.common.constant.message.push.PushDeviceType;
import com.starbuds.server.common.constant.message.push.PushTarget;
import com.starbuds.server.common.constant.message.push.PushType;
import com.starbuds.server.common.pojo.db.PushSubTaskDO;
import com.starbuds.server.common.pojo.message.push.PushMsg;
import com.starbuds.server.common.pojo.message.push.PushMsgParam;
import com.starbuds.server.common.pojo.mq.QueuePushTaskAddMsg;
import com.starbuds.server.common.service.core.IdentityService;
import com.starbuds.server.common.service.message.PushService;
import com.starbuds.server.service.task.dao.PushDeviceDAO;
import com.starbuds.server.service.task.dao.PushSubTaskDAO;
import com.starbuds.server.service.task.dao.PushTaskDAO;
import com.starbuds.server.service.task.dao.UserDAO;
import com.starbuds.server.service.task.entity.PushDeviceDTO;
import com.starbuds.server.service.task.entity.PushTaskDTO;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Author jiangxiaowei
 * @DateTime 2020/7/6 15:27
 * @Description 推送任务
 */
@Slf4j
@Component
public class PushTaskProvider {

    @Resource
    PushTaskDAO pushTaskDAO;

    @Reference(version = "1.0.0")
    IdentityService identityService;

    @Resource
    UserDAO userDAO;

    @Resource
    PushDeviceDAO pushDeviceDAO;

    @Resource
    PushSubTaskDAO pushSubTaskDAO;

    @Reference(version = "1.0.0")
    PushService pushService;

    // 单次推送500条
    public static final int pushNum = 500;

    /**
     * 处理推送任务添加
     *
     * @param msg
     */
    @Transactional(rollbackFor = Exception.class)
    public void handlePushTaskAdd(QueuePushTaskAddMsg msg) {
        Long taskId = msg.getTaskId();
        log.info("处理推送任务，taskId：{}", taskId);

        long curTime = System.currentTimeMillis();

        PushTaskDTO taskDTO = pushTaskDAO.selectTask(taskId);
        if (taskDTO == null) {
            log.info("任务不存在");
            return;
        }

        log.info("任务详情：{}", JSON.toJSONString(taskDTO));

        if (pushTaskDAO.updateStatus(taskId, TaskStatusEnum.IN_PROGRESS.getValue(), TaskStatusEnum.WAIT.getValue()) > 0) {

            TaskTypeEnum taskType = TaskTypeEnum.toEnum(taskDTO.getTaskType());
            // 目标
            String targetValue = taskDTO.getTargetValue();

            PushMsg pushMsg = new PushMsg();
            pushMsg.setDeviceType(PushDeviceType.All);
            pushMsg.setPushType(PushType.Notice);
            pushMsg.setTitle(taskDTO.getPushTitle());
            pushMsg.setBody(taskDTO.getPushBody());

            String extParameters = taskDTO.getExtParameters();

            pushMsg.setParam(JSON.parseObject(extParameters, PushMsgParam.class));


            if (TaskTypeEnum.ALL_PUSH == taskType) {
                pushMsg.setPushTarget(PushTarget.All);
                pushMsg.setTargetValue(PushTarget.All.getValue());
                // 所有包
                List<String> packageNames = pushDeviceDAO.selectPackageNames();
                List<PushSubTaskDO> subTaskList = new ArrayList<>();
                Long[] taskIds = identityService.getIds(packageNames.size());
                int pushTotal = pushDeviceDAO.countAll();
                if (pushTotal == 0) {
                    log.info("暂无设备注册");
                    // 更新数量
                    pushTaskDAO.updateTaskNum(taskId, 0, 0, 0, 0);
                    // 更新为执行失败
                    pushTaskDAO.updatePushFailed(taskId, TaskStatusEnum.FAILED.getValue(), "推送失败(暂无设备注册)");
                    return;
                }

                for (int i = 0; i < packageNames.size(); i++) {
                    String packageName = packageNames.get(i);

                    PushSubTaskDO subTask = new PushSubTaskDO();
                    subTask.setTaskId(taskId);
                    subTask.setSubId(taskIds[i]);
                    subTask.setTotalNum(pushDeviceDAO.countByPackageName(packageName));
                    subTask.setTargetValue(PushTarget.All.getValue());
                    subTask.setExecuteTime(curTime);
                    subTask.setTaskCreateTime(curTime);
                    subTask.setTaskStatus(TaskStatusEnum.SUCCESS.getValue());
                    subTask.setPackageName(packageName);
                    subTaskList.add(subTask);

                    boolean result = pushService.pushMsg(pushMsg, packageName);
                    log.info("包名{}，推送结果{}", packageName, result);

                }
                // 子任务
                pushSubTaskDAO.batchInsert(subTaskList);
                pushTaskDAO.updateTaskNum(taskId, pushTotal, subTaskList.size(), pushTotal, subTaskList.size());
                // 更新为执行成功
                pushTaskDAO.updatePushSuccess(taskId, TaskStatusEnum.SUCCESS.getValue());
                return;
            } else {
                pushMsg.setPushTarget(PushTarget.Device);

                // 推送的用户
                List<Long> userIds = getUserIds(taskId, taskType, targetValue);
                log.info("共需推送的用户数量:{}", userIds.size());
                // 对应的设备id
                if (CollectionUtils.isEmpty(userIds)) {
                    log.info("未匹配到用户");
                    // 更新数量
                    pushTaskDAO.updateTaskNum(taskId, 0, 0, 0, 0);
                    // 更新为执行失败
                    pushTaskDAO.updatePushFailed(taskId, TaskStatusEnum.FAILED.getValue(), "推送失败(未匹配到用户)");
                    return;
                }
                List<PushDeviceDTO> deviceList = pushDeviceDAO.selectDevices(userIds);
                log.info("共需推送{}个设备", deviceList.size());
                if (CollectionUtils.isEmpty(deviceList)) {
                    log.info("未匹配到推送的设备,用户id{}", userIds);
                    // 更新数量
                    pushTaskDAO.updateTaskNum(taskId, 0, 0, 0, 0);
                    // 更新为执行失败
                    pushTaskDAO.updatePushFailed(taskId, TaskStatusEnum.FAILED.getValue(), "推送失败(未匹配到推送的设备)");
                    return;
                }
                Map<String, List<PushDeviceDTO>> packageNameMap = deviceList.stream().collect(Collectors.groupingBy(PushDeviceDTO::getPackageName));

                int pushTotal = deviceList.size();
                int pushSub = 0;

                for (Map.Entry<String, List<PushDeviceDTO>> entry : packageNameMap.entrySet()) {
                    // 包名
                    String packageName = entry.getKey();
                    // 设备对象
                    List<PushDeviceDTO> entryValue = entry.getValue();
                    // 设备
                    List<String> deviceIds = entryValue.stream().map(PushDeviceDTO::getDeviceId).collect(Collectors.toList());
                    List<Long> userIdList = entryValue.stream().map(PushDeviceDTO::getUserId).collect(Collectors.toList());
                    int totalNum = deviceIds.size();
                    int subNum = getSubNum(pushNum, totalNum);
                    // 总的子任务数
                    pushSub += subNum;

                    log.info("taskId:{},包名:{},共需推送{}个设备，共{}个子任务", taskId, packageName, totalNum, subNum);

                    // 子任务
                    List<PushSubTaskDO> subTaskList = new ArrayList<>();
                    Long[] taskIds = identityService.getIds(subNum);
                    for (int i = 0; i < subNum; i++) {

                        int pageStart = i * pushNum;
                        int pageEnd = (i + 1) * pushNum + 1;
                        if (pageEnd > totalNum) {
                            pageEnd = totalNum;
                        }

                        PushSubTaskDO subTask = new PushSubTaskDO();
                        subTask.setTaskId(taskId);
                        subTask.setSubId(taskIds[i]);
                        subTask.setTotalNum(i == subNum - 1 ? totalNum % pushNum : pushNum);
                        subTask.setTargetValue(CollectionUtil.join(userIdList.subList(pageStart, pageEnd), ","));
                        subTask.setExecuteTime(curTime);
                        subTask.setTaskCreateTime(curTime);
                        subTask.setTaskStatus(TaskStatusEnum.SUCCESS.getValue());
                        subTask.setPackageName(packageName);
                        subTaskList.add(subTask);
                        String msgTargetValue = CollectionUtil.join(deviceIds.subList(pageStart, pageEnd), ",");
                        pushMsg.setTargetValue(msgTargetValue);
                        boolean result = pushService.pushMsg(pushMsg, packageName);
                        log.info("包名{}，推送结果{}", packageName, result);
                    }
                    // 同包名一起存
                    pushSubTaskDAO.batchInsert(subTaskList);
                }
                // 更新数量
                pushTaskDAO.updateTaskNum(taskId, pushTotal, pushSub, pushTotal, pushSub);
                // 更新为执行成功
                pushTaskDAO.updatePushSuccess(taskId, TaskStatusEnum.SUCCESS.getValue());
                return;
            }
        }
        log.info("任务状态已变更，不再执行");
    }

    /**
     * 获取用户id
     *
     * @param taskId
     * @param taskType
     * @param targetValue
     * @return
     */
    private List<Long> getUserIds(Long taskId, TaskTypeEnum taskType, String targetValue) {
        // 推送的目标用户
        List<Long> userIds = new ArrayList<>();
        switch (taskType) {
            case ALL_PUSH:
                log.info("全员发送,taskId:{}", taskId);
                break;
            case SINGLE_PUSH:
                log.info("单推{}", targetValue);
                userIds.add(Convert.toLong(targetValue));
                break;
            case BATCH_PUSH:
                log.info("批量推送{}", targetValue);
                userIds.addAll(Arrays.stream(targetValue.split(",")).map(e -> Convert.toLong(e)).collect(Collectors.toList()));
                break;
            case SEX_PUSH:
                //  0,"未知" 1,"男" 2,"女"
                log.info("按性别推送{}", targetValue);
                Set<Integer> sexSet = Arrays.stream(targetValue.split(",")).map(e -> Convert.toInt(e)).collect(Collectors.toSet());
                userIds.addAll(userDAO.selectUsers(sexSet, null, null, null, null, null, null, null, null));
                break;
            case AREA_PUSH:
                log.info("按地区推送{}", targetValue);
                Set<Integer> areaSet = Arrays.stream(targetValue.split(",")).map(e -> Convert.toInt(e)).collect(Collectors.toSet());
                userIds.addAll(userDAO.selectUsers(null, areaSet, null, null, null, null, null, null, null));
                break;
            case CHANNEL_PUSH:
                log.info("按渠道推送{}", targetValue);
                Set<Long> channelSet = Arrays.stream(targetValue.split(",")).map(e -> Convert.toLong(e)).collect(Collectors.toSet());
                userIds.addAll(userDAO.selectUsers(null, null, channelSet, null, null, null, null, null, null));
                break;
            case USER_LEVEL_PUSH:
                log.info("按用户等级推送{}", targetValue);
                Set<Integer> wealthLevelSet = Arrays.stream(targetValue.split(",")).map(e -> Convert.toInt(e)).collect(Collectors.toSet());
                userIds.addAll(userDAO.selectUsers(null, null, null, wealthLevelSet, null, null, null, null, null));
                break;
            case ANCHOR_LEVEL_PUSH:
                log.info("按主播等级推送{}", targetValue);
                Set<Integer> charmLevelSet = Arrays.stream(targetValue.split(",")).map(e -> Convert.toInt(e)).collect(Collectors.toSet());
                userIds.addAll(userDAO.selectUsers(null, null, null, null, charmLevelSet, null, null, null, null));
                break;
            case ANCHOR_PUSH:
                // 0直播主播  1单聊主播
                log.info("按主播推送{}", targetValue);
                Integer anchorType = Convert.toInt(targetValue);
                userIds.addAll(userDAO.selectUsers(null, null, null, null, null, anchorType, null, null, null));
                break;
            case NOT_ANCHOR_PUSH:
                // -1 非主播用户
                log.info("按非主播推送{}", targetValue);
                Integer isAnchor = 0;
                userIds.addAll(userDAO.selectUsers(null, null, null, null, null, null, isAnchor, null, null));
                break;
            case HAS_RECHARGE_PUSH:
                // 0未充值  1已充值
                log.info("按是否充值推送{}", targetValue);
                Integer hasRecharged = Convert.toInt(targetValue);
                userIds.addAll(userDAO.selectUsers(null, null, null, null, null, null, null, hasRecharged, null));
                break;
            case USER_IDENTITY_PUSH:
                // 0普通用户  1会员   2贵族
                log.info("按用户身份推送{}", targetValue);
                Integer userIdentity = Convert.toInt(targetValue);
                userIds.addAll(userDAO.selectUsers(null, null, null, null, null, null, null, null, userIdentity));
                break;
            case MULTI_CONDITION_PUSH:
                log.info("按多条件推送{}", targetValue);
                // 条件以;分隔，筛选出用户求交集
                Set<String> multiCondition = Arrays.stream(targetValue.split(";")).collect(Collectors.toSet());
                Set<Integer> muSex = new HashSet<>();
                Set<Integer> muArea = new HashSet<>();
                Set<Long> muChannel = new HashSet<>();
                Set<Integer> muWealthLevel = new HashSet<>();
                Set<Integer> muCharmLevel = new HashSet<>();
                Integer muAnchorType = null;
                Integer muIsAnchor = null;
                Integer muHasRecharged = null;
                Integer muUserIdentity = null;
                for (String item : multiCondition) {
                    // 类型:值
                    String[] strArray = item.split(":");
                    if (strArray.length != 2) {
                        log.info("参数不正确，targetValue{},item{}", targetValue, item);
                        continue;
                    }

                    taskType = TaskTypeEnum.toEnum(Convert.toInt(strArray[0]));
                    targetValue = strArray[1];

                    switch (taskType) {
                        case SEX_PUSH:
                            log.info("按性别推送{}", targetValue);
                            muSex = Arrays.stream(targetValue.split(",")).map(e -> Convert.toInt(e)).collect(Collectors.toSet());
                            break;
                        case AREA_PUSH:
                            log.info("按地区推送{}", targetValue);
                            muArea = Arrays.stream(targetValue.split(",")).map(e -> Convert.toInt(e)).collect(Collectors.toSet());
                            break;
                        case CHANNEL_PUSH:
                            log.info("按渠道推送{}", targetValue);
                            muChannel = Arrays.stream(targetValue.split(",")).map(e -> Convert.toLong(e)).collect(Collectors.toSet());
                            break;
                        case USER_LEVEL_PUSH:
                            log.info("按用户等级推送{}", targetValue);
                            muWealthLevel = Arrays.stream(targetValue.split(",")).map(e -> Convert.toInt(e)).collect(Collectors.toSet());
                            break;
                        case ANCHOR_LEVEL_PUSH:
                            log.info("按主播等级推送{}", targetValue);
                            muCharmLevel = Arrays.stream(targetValue.split(",")).map(e -> Convert.toInt(e)).collect(Collectors.toSet());
                            break;
                        case ANCHOR_PUSH:
                            log.info("按主播推送{}", targetValue);
                            muAnchorType = Convert.toInt(targetValue);
                            break;
                        case NOT_ANCHOR_PUSH:
                            log.info("按非主播推送{}", targetValue);
                            muIsAnchor = 0;
                            break;
                        case HAS_RECHARGE_PUSH:
                            log.info("按是否充值推送{}", targetValue);
                            muHasRecharged = Convert.toInt(targetValue);
                            break;
                        case USER_IDENTITY_PUSH:
                            log.info("按用户身份推送{}", targetValue);
                            muUserIdentity = Convert.toInt(targetValue);
                            break;
                        case ALL_PUSH:
                            log.info("此处不应该有全员推送");
                            break;
                        default:
                            break;
                    }
                }
                return userDAO.selectUsers(muSex, muArea, muChannel, muWealthLevel, muCharmLevel, muAnchorType, muIsAnchor, muHasRecharged, muUserIdentity);
            default:
                break;
        }
        return userIds;
    }

    /**
     * 判断子任务数量
     *
     * @param pushNum
     * @param totalNum
     * @return
     */
    private int getSubNum(int pushNum, int totalNum) {
        int subNum = totalNum / pushNum;
        int remainder = totalNum % pushNum;
        if (remainder > 0) {
            subNum += 1;
        }
        return subNum;
    }


}
