package com.starbuds.server.service.task.provider;

import cn.hutool.crypto.SecureUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.google.common.base.Preconditions;
import com.starbuds.server.common.utils.LogUtil;
import com.starbuds.server.service.task.util.HttpUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.RandomUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.math.NumberUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.PostConstruct;
import java.util.*;

import static com.starbuds.server.service.task.util.SpringContextUtil.isDev;

/**
 * 融云服务
 *
 * @author aoe
 * @date 2021/10/9
 */
@Service
@Slf4j
public class RongCloudProvider {

    @Value("${im.rongcloud.appKey}")
    private String appKey;

    @Value("${im.rongcloud.appSecret}")
    private String appSecret;

    /**
     * 融云数据中心地址
     * 受到攻击时自动切换（出现频繁超时）
     */
    private String[] rongCloudDomains;

    /**
     * 融云数据中心地址索引
     */
    private int rongCloudDomainIndex;

    @PostConstruct
    private void init() {
        this.rongCloudDomains = new String[]{
                "https://api-cn.ronghub.com",
                "https://api2-cn.ronghub.com"
        };
        // 默认使用主地址
        this.rongCloudDomainIndex = 0;
    }

    /**
     * 获得融云离线用户列表
     * https://doc.rongcloud.cn/imserver/server/v1/chatroom/info#users_exist
     *
     * @param roomId
     * @param userIds
     * @return
     */
    public List<Long> getImOfflineUserIds(long roomId, List<Long> userIds) {
        if (CollectionUtils.isEmpty(userIds)) {
            return Collections.emptyList();
        }
        int size = userIds.size();
        Preconditions.checkArgument(size <= 1000, "userIds 最大数量1000，当前数量" + size);

        String url = rongCloudDomain() + "/chatroom/users/exist.json";
        Map<String, String> body = new HashMap<>(size);
        body.put("chatroomId", "" + roomId);
        String userIdKey = "userId";
        userIds.stream()
                .filter(Objects::nonNull)
                .forEach(userId -> body.put(userIdKey, userId + ""));
        String rsp = HttpUtil.postForm(url, body, getRongHeaders());

        if (isDev()) {
            log.info(LogUtil.devLog("获得融云离线用户列表 roomId={} rsp={}"), roomId, rsp);
        }

        if (StringUtils.isBlank(rsp)) {
            log.error("获得融云离线用户列表失败，返回结果 is blank roomId=" + roomId);
            return Collections.emptyList();
        }
        if ("timeout".equals(rsp)) {
            changeRongCloudDomain();
            return Collections.emptyList();
        }

        JSONObject json = JSON.parseObject(rsp);
        if (json == null) {
            log.error("获得融云离线用户列表失败，返回结果不是JSON格式，roomId=" + roomId + " rsp=" + rsp);
            return Collections.emptyList();
        }

        int code = json.getIntValue("code");
        if (code - 200 != 0) {
            log.error("获得融云离线用户列表失败，code != 200，roomId={} roomId=" + roomId + " rsp=" + rsp);
            return Collections.emptyList();
        }
        JSONArray result = json.getJSONArray("result");
        if (result == null) {
            log.error("获得融云离线用户列表失败，result is null，roomId={} roomId=" + roomId + " rsp=" + rsp);
            return Collections.emptyList();
        }

        List<Long> offLineUserIds = new ArrayList<>();
        for (int i = 0; i < result.size(); i++) {
            JSONObject user = result.getJSONObject(i);
            if (user == null) {
                continue;
            }

            int isInChrm = user.getIntValue("isInChrm");
            Long userId = NumberUtils.toLong(user.getString("userId"));

            if (userId > 0 && isInChrm == 0) {
                offLineUserIds.add(userId);
            }
        }

        if (isDev()) {
            log.info(LogUtil.devLog("融云离线用户 roomId={} {}"), roomId, offLineUserIds);
        }
        return offLineUserIds;
    }

    /**
     * 融云请求头
     *
     * @return
     */
    private Map<String, String> getRongHeaders() {
        String nonce = RandomUtils.nextInt(1000, 9999) + "";
        String timestamp = System.currentTimeMillis() + "";
        String toSign = String.format("%s%s%s", appSecret, nonce, timestamp);
        String sign = SecureUtil.sha1(toSign);

        Map<String, String> headers = new HashMap<>();
        headers.put("RC-App-Key", appKey);
        headers.put("RC-Nonce", nonce);
        headers.put("RC-Timestamp", timestamp);
        headers.put("RC-Signature", sign);

        return headers;
    }

    /**
     * 融云服务地址
     *
     * @return
     */
    private String rongCloudDomain() {
        return rongCloudDomains[rongCloudDomainIndex];
    }

    /**
     * 切换融云服务地址
     */
    private void changeRongCloudDomain() {
        if (0 == rongCloudDomainIndex) {
            rongCloudDomainIndex = 1;
        } else if (1 == rongCloudDomainIndex) {
            rongCloudDomainIndex = 0;
        }
        log.info("切换融云服务地址 rongCloudDomainIndex={}", rongCloudDomainIndex);
    }
}
