package com.starbuds.server.service.task.provider;

import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.constant.activity.LotteryRewardTypeEnum;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.constant.finance.BagItemTypeEnum;
import com.starbuds.server.common.constant.finance.WalletTypeEnum;
import com.starbuds.server.common.constant.statistics.StatPlatformTypeEnum;
import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.common.pojo.daoplus.QueryWrapper;
import com.starbuds.server.common.pojo.daoplus.UpdateWrapper;
import com.starbuds.server.common.pojo.db.StatGoldCoinDO;
import com.starbuds.server.common.pojo.db.StatGoldCoinExchangeDO;
import com.starbuds.server.common.pojo.task.qo.StatGoldCoinPageListQO;
import com.starbuds.server.common.service.core.IdentityService;
import com.starbuds.server.common.utils.DateUtil;
import com.starbuds.server.service.task.dao.CoinGoodsOrderDAO;
import com.starbuds.server.service.task.dao.StatGoldCoinDAO;
import com.starbuds.server.service.task.dao.WishRewardHisDAO;
import com.starbuds.server.service.task.entity.stat.StatDayPretty;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.math.NumberUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;

import javax.annotation.Resource;
import java.time.LocalDate;
import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

import static com.starbuds.server.common.pojo.util.PageListUtil.buildPageListQueryWrapper;

/**
 * 金币统计
 */
@Component
@Slf4j
public class StatGoldCoinProvider {

    /**
     * 主键
     */
    private String PRIMARY_KEY = "statGoldCoinId";

    @Reference(version = "1.0.0")
    private IdentityService identityService;

    @Resource
    private StatGoldCoinDAO statGoldCoinDAO;

    @Resource
    private StatPlatformProvider statPlatformProvider;

    @Resource
    private CoinGoodsOrderDAO coinGoodsOrderDAO;

    @Resource
    private StatGoldCoinExchangeProvider statGoldCoinExchangeProvider;

    @Resource
    private WishRewardHisDAO wishRewardHisDAO;

    /**
     * 插入数据
     *
     * @param po
     * @return null:插入失败
     */
    @Transactional(rollbackFor = Exception.class)
    public StatGoldCoinDO create(StatGoldCoinDO po) {
        po.setStatGoldCoinId(identityService.getId());
        po.setCreateTime(System.currentTimeMillis());
        po.setUpdateTime(po.getCreateTime());
        po.setIsDeleted(BoolIntVal.False);
        boolean flag = statGoldCoinDAO.insert(po) == 1;

        String json = JSON.toJSONString(po);
        if (flag) {
            log.info("金币统计创建成功 {}", json);
            return po;
        }
        log.warn("金币统计创建失败 {}", json);
        return null;
    }

    /**
     * 插入批量数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean createBatch(List<StatGoldCoinDO> list) {
        boolean flag = statGoldCoinDAO.insertBatch(list) > 0;
        return flag;
    }

    /**
     * 插入或更新数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean createOrUpdate(StatGoldCoinDO po) {
        boolean flag = statGoldCoinDAO.insertOrUpdate(po) == 1;
        return flag;
    }

    /**
     * 更新数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean updateById(StatGoldCoinDO po) {
        po.setCreateTime(null);
        po.setUpdateTime(System.currentTimeMillis());

        UpdateWrapper uw = new UpdateWrapper();
        uw.setPo(po);
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, po.getStatGoldCoinId());
        uw.setQw(qw);
        return update(uw);
    }

    /**
     * 更新数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean update(UpdateWrapper uw) {
        boolean flag = statGoldCoinDAO.update(uw) > 0;
        return flag;
    }

    /**
     * 根据id查询
     */
    public StatGoldCoinDO getById(Long id) {
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, id);
        return statGoldCoinDAO.selectOne(qw);
    }

    /**
     * 判断是否存在
     *
     * @param id 主键
     * @return true:存在
     */
    public boolean exist(Long id) {
        if (id == null) {
            return false;
        }
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, id);
        return statGoldCoinDAO.selectCount(qw) > 0;
    }

    /**
     * 分页查询
     */
    public PageList<StatGoldCoinDO> getPageList(StatGoldCoinPageListQO qo) {
        QueryWrapper qw = buildPageListQueryWrapper(qo);
        PageList<StatGoldCoinDO> page = new PageList<>(qo.getPageIndex(), qo.getPageSize());
        qw.setPageCursor(page.getPageCursor());
        qw.setPageSize(page.getPageSize());
        page.setList(statGoldCoinDAO.selectPageList(qw));
        page.setTotal(statGoldCoinDAO.selectCount(qw));
        return page;
    }

    /**
     * 根据ids获得 DO Map
     *
     * @param ids id集合
     * @return Map<主键, DO对象>
     */
    public Map<Long, StatGoldCoinDO> getMapByIds(Collection ids) {
        return statGoldCoinDAO.selectMapByIds(ids);
    }

    // 手动编码开始

    /**
     * 根据日期统计金币数据
     *
     * @param date 统计日期
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean statByDate(LocalDate date) {
        int statTime = DateUtil.toYearMonthDay(date);
        long data = NumberUtils.toLong("" + statTime);
        StatDayPretty totalIssue = statPlatformProvider.getStatByYearMonthDay(data, StatPlatformTypeEnum.GOLD_COIN_ADD);
        StatDayPretty totalCost = statPlatformProvider.getStatByYearMonthDay(data, StatPlatformTypeEnum.GOLD_COIN_REDUCE);

        Long startTime = DateUtil.getDayBegin(date);
        Long endTime = DateUtil.getDayEnd(date);

        StatGoldCoinDO po = new StatGoldCoinDO();

        // 因为是SUM求和, 可能为空
        Long todayTotalIssueWish = wishRewardHisDAO.getWishRewardStat(BagItemTypeEnum.GOLDEN_COIN.getValue(), startTime, endTime);
        if (todayTotalIssueWish == null) {
            todayTotalIssueWish = 0L;
        }

        po.setTodayTotalIssueWish(todayTotalIssueWish);
        po.setStatTime(statTime);
        po.setTodayTotalIssue(totalIssue.getValue());
        po.setTodayTotalCost(totalCost.getValue());
        // 今日结余金币 = 今日发行金币总量 - 今日用户消耗金币总量
        Long todayBalance = po.getTodayTotalIssue() - po.getTodayTotalCost();
        po.setTodayBalance(todayBalance);

        // 平台总结余金币 = 前一日平台总结余金币 + 今日结余
        int beforeDay = DateUtil.toYearMonthDay(date.minusDays(1));
        QueryWrapper qw = new QueryWrapper();
        qw.eq("statTime", beforeDay);
        qw.eq("isDeleted", BoolIntVal.False);
        StatGoldCoinDO beforeStat = statGoldCoinDAO.selectOne(qw);
        Long totalBalance = todayBalance;
        if (beforeStat != null) {
            totalBalance += beforeStat.getTotalBalance();
        }
        po.setTotalBalance(totalBalance);


        int priceType = WalletTypeEnum.GOLD_COIN.getValue();
        // 今日消耗金币的用户数量
        po.setTodayUserCount(coinGoodsOrderDAO.selectUserCount(startTime, endTime, priceType));

        // 清空之前统计的数据
        QueryWrapper upQw = new QueryWrapper();
        upQw.eq("statTime", statTime);
        StatGoldCoinDO upPo = new StatGoldCoinDO();
        upPo.setIsDeleted(BoolIntVal.True);
        UpdateWrapper uw = new UpdateWrapper();
        uw.setQw(upQw);
        uw.setPo(upPo);
        int clearCount = statGoldCoinDAO.update(uw);
        if (clearCount > 0) {
            log.info("清空之前金币统计数据 statTime={} 共{}条", statTime, clearCount);
        }

        // 保存金币统计数据
        StatGoldCoinDO createStatGoldCoin = create(po);
        String poJson = JSON.toJSONString(po);
        if (createStatGoldCoin == null) {
            log.error("保存金币统计数据失败 {}", poJson);
            return false;
        }

        // 保存金币兑换物品统计数据
        List<StatGoldCoinExchangeDO> exchanges = coinGoodsOrderDAO.selectExchangeItems(startTime, endTime, priceType);
        if (CollectionUtils.isEmpty(exchanges)) {
            log.info("保存金币统计数据成功，没有兑换商品 {}", poJson);
            return true;
        }

        exchanges = exchanges.stream()
                .filter(Objects::nonNull)
                .map(item -> {
                    item.setStatGoldCoinId(createStatGoldCoin.getStatGoldCoinId());
                    item.setCreateTime(System.currentTimeMillis());
                    item.setUpdateTime(item.getCreateTime());
                    item.setIsDeleted(BoolIntVal.False);
                    return item;
                })
                .collect(Collectors.toList());
        String exchangesJson = JSON.toJSONString(exchanges);
        boolean saveExchanges = statGoldCoinExchangeProvider.createBatch(exchanges);
        if (Boolean.FALSE.equals(saveExchanges)) {
            log.error("保存金币兑换物品统计数据失败 {}", exchangesJson);
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return false;
        }
        log.info("保存金币兑换物品统计数据成功 {}", exchangesJson);
        return true;
    }

    public Long getFirstTime() {
        return statGoldCoinDAO.getFirstTime();
    }
}