package com.starbuds.server.service.task.job;

import cn.hutool.core.io.FileUtil;
import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.constant.task.ExportExcelDimensionEnum;
import com.starbuds.server.common.pojo.daoplus.QueryWrapper;
import com.starbuds.server.common.pojo.daoplus.UpdateWrapper;
import com.starbuds.server.common.pojo.db.ExcelExportDO;
import com.starbuds.server.common.service.core.IdentityService;
import com.starbuds.server.common.utils.DateUtil;
import com.starbuds.server.common.constant.task.ExportExcelBizTypeEnum;
import com.starbuds.server.service.task.constant.JobHandlers;
import com.starbuds.server.service.task.dao.ExcelExportDAO;
import com.starbuds.server.service.task.entity.ExportExcelDTO;
import com.starbuds.server.service.task.provider.ExcelProvider;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import java.io.File;
import java.time.LocalDate;
import java.util.*;


/**
 * 生成指定时间的Excel
 * Excel文件查询：业务类型、名称、时间、导出纬度（日、月、周）
 *
 * @author laoliu
 * @date 2021/08/25
 */
@Component
@Slf4j
public class ExportExcelJob {

    /**
     * 日志前缀
     */
    private final String LOG_PREFIX = "【导出数据持久化处理】";

    private Map<Integer, ExportExcel> exportExcelBeanMap = new HashMap<>();

    @Value("${excel.path}")
    private String path;

    @Resource
    ApplicationContext context;

    @Resource
    ExcelExportDAO excelExportDAO;

    @Reference(version = "1.0.0")
    IdentityService identityService;

    @Resource
    ExcelProvider excelProvider;

    @PostConstruct
    private void init() {
        // 初始化所有实现了ExportExcel接口的实现类
        Map<String, ExportExcel> map = context.getBeansOfType(ExportExcel.class);
        map.forEach((key, value) -> exportExcelBeanMap.put(value.getBizType().getValue(), value));
    }


    /**
     * （Excel文件查询：业务类型、名称、时间、导出纬度 日
     *
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @XxlJob(value = JobHandlers.EXPORT_EXCEL_JOB_DAILY_TASK)
    public ReturnT<String> exportExcelJobDailyHandler(String param) {
        // 参数校验
        log.info(LOG_PREFIX + "入口参数: {}", param);
        if (StringUtils.isBlank(param)) {
            log.warn(LOG_PREFIX + "入口参数 {} 为空，不做记录", param);
            return ReturnT.SUCCESS;
        }

        ExportExcelDTO paramObject = JSON.parseObject(param, ExportExcelDTO.class);

        if (paramObject == null) {
            log.warn(LOG_PREFIX + "入口参数 {} 解析失败，不做记录", param);
            return ReturnT.SUCCESS;
        }

        Integer bizType = paramObject.getBizType();
        if (bizType == null) {
            log.warn(LOG_PREFIX + "入口参数bizType解析失败，不做记录: {}", param);
            return ReturnT.SUCCESS;
        }

        ExportExcelBizTypeEnum bizTypeEnum = ExportExcelBizTypeEnum.toEnum(paramObject.getBizType());
        if (bizTypeEnum == null) {
            log.warn(LOG_PREFIX + "业务类型不存在，不做记录: {}", param);
            return ReturnT.SUCCESS;
        }

        // 获取实现类
        ExportExcel exportExcel = exportExcelBeanMap.get(bizTypeEnum.getValue());

        Long startTime = paramObject.getStartTime() == null ? DateUtil.getDayBegin(DateUtil.getYesterday(new Date())) : paramObject.getStartTime();
        Long endTime = paramObject.getEndTime() == null ? DateUtil.getDayEnd(DateUtil.getYesterday(new Date())) : paramObject.getEndTime();

        return operateFile(exportExcel, paramObject, startTime, endTime, ExportExcelDimensionEnum.DAY) ? ReturnT.SUCCESS : ReturnT.FAIL;
    }


    /**
     * （Excel文件查询：业务类型、名称、时间、导出纬度 周
     *
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @XxlJob(value = JobHandlers.EXPORT_EXCEL_JOB_WEEKLY_TASK)
    public ReturnT<String> exportExcelJobWeeklyHandler(String param) {
        // 参数校验
        log.info(LOG_PREFIX + "入口参数: {}", param);
        if (StringUtils.isBlank(param)) {
            log.warn(LOG_PREFIX + "入口参数 {} 为空，不做记录", param);
            return ReturnT.SUCCESS;
        }

        ExportExcelDTO paramObject = JSON.parseObject(param, ExportExcelDTO.class);

        if (paramObject == null) {
            log.warn(LOG_PREFIX + "入口参数 {} 解析失败，不做记录", param);
            return ReturnT.SUCCESS;
        }

        Integer bizType = paramObject.getBizType();
        if (bizType == null) {
            log.warn(LOG_PREFIX + "入口参数bizType解析失败，不做记录: {}", param);
            return ReturnT.SUCCESS;
        }

        ExportExcelBizTypeEnum bizTypeEnum = ExportExcelBizTypeEnum.toEnum(paramObject.getBizType());
        if (bizTypeEnum == null) {
            log.warn(LOG_PREFIX + "业务类型不存在，不做记录: {}", param);
            return ReturnT.SUCCESS;
        }

        // 获取实现类
        ExportExcel exportExcel = exportExcelBeanMap.get(bizTypeEnum.getValue());

        Long startTime = paramObject.getStartTime() == null ? DateUtil.getLastWeekStartTimeStamp() : paramObject.getStartTime();
        Long endTime = paramObject.getEndTime() == null ? DateUtil.getLastWeekEndTimeStamp() : paramObject.getEndTime();

        return operateFile(exportExcel, paramObject, startTime, endTime, ExportExcelDimensionEnum.WEEK) ? ReturnT.SUCCESS : ReturnT.FAIL;
    }


    /**
     * （Excel文件查询：业务类型、名称、时间、导出纬度 月
     *
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @XxlJob(value = JobHandlers.EXPORT_EXCEL_JOB_MONTH_TASK)
    public ReturnT<String> exportExcelJobMonthHandler(String param) {
        // 参数校验
        log.info(LOG_PREFIX + "入口参数: {}", param);
        if (StringUtils.isBlank(param)) {
            log.warn(LOG_PREFIX + "入口参数 {} 为空，不做记录", param);
            return ReturnT.SUCCESS;
        }

        ExportExcelDTO paramObject = JSON.parseObject(param, ExportExcelDTO.class);

        if (paramObject == null) {
            log.warn(LOG_PREFIX + "入口参数 {} 解析失败，不做记录", param);
            return ReturnT.SUCCESS;
        }

        Integer bizType = paramObject.getBizType();
        if (bizType == null) {
            log.warn(LOG_PREFIX + "入口参数bizType解析失败，不做记录: {}", param);
            return ReturnT.SUCCESS;
        }

        ExportExcelBizTypeEnum bizTypeEnum = ExportExcelBizTypeEnum.toEnum(paramObject.getBizType());
        if (bizTypeEnum == null) {
            log.warn(LOG_PREFIX + "业务类型不存在，不做记录: {}", param);
            return ReturnT.SUCCESS;
        }

        // 获取实现类
        ExportExcel exportExcel = exportExcelBeanMap.get(bizTypeEnum.getValue());

        Long startTime = paramObject.getStartTime() == null ? DateUtil.getLastMonthStart(new Date()) : paramObject.getStartTime();
        Long endTime = paramObject.getEndTime() == null ? DateUtil.getLastMonthEnd(new Date()) : paramObject.getEndTime();

        return operateFile(exportExcel, paramObject, startTime, endTime, ExportExcelDimensionEnum.MONTH) ? ReturnT.SUCCESS : ReturnT.FAIL;
    }


    public boolean operateFile(ExportExcel exportExcel, ExportExcelDTO paramObject, Long startTime, Long endTime, ExportExcelDimensionEnum dimension) {
        // 如果文件已存在
        // fix - 去数据库查询是否存在
        QueryWrapper wrapper = new QueryWrapper();
        wrapper.eq("path", excelProvider.getDir() + getPath(exportExcel.getFileName(), DateUtil.getIntDate(new Date(startTime)), dimension.getTable()));
        wrapper.eq("bizType", exportExcel.getBizType().getValue());

        if (excelExportDAO.selectCount(wrapper) > 0) {
            // 不强制覆盖
            if (paramObject.getIsForceCover() != null && paramObject.getIsForceCover() == 0) {
                return true;
            }
            // 强制覆盖
            return update(exportExcel, startTime, endTime, dimension, paramObject.getRemark());
        } else {

            // 文件不存在
            return insert(exportExcel, startTime, endTime, dimension, paramObject.getRemark());
        }
    }


    private boolean insert(ExportExcel exportExcel, Long startTime, Long endTime, ExportExcelDimensionEnum dimension, String remark) {
        // 计算耗时
        long useTimeStart = System.currentTimeMillis();

        List list = Optional.ofNullable(exportExcel.getRowsList(startTime, endTime)).orElse(new ArrayList());

        String filename = getPath(exportExcel.getFileName(), DateUtil.getIntDate(new Date(startTime)), dimension.getTable());

        // 保存到磁盘
        excelProvider.save2Disk(filename, exportExcel.getSheetName(), list, exportExcel.getClassType());

        long endTimeStart = System.currentTimeMillis();

        ExcelExportDO exportDO = new ExcelExportDO();
        exportDO.setExcelId(identityService.getId());
        exportDO.setExcelName(exportExcel.getFileName());
        exportDO.setBizName(exportExcel.getBizType().getLabel());
        exportDO.setBizType(exportExcel.getBizType().getValue());

        exportDO.setDate(DateUtil.dateToLocalDate(new Date(startTime)).getDayOfMonth());
        exportDO.setMonth(DateUtil.dateToLocalDate(new Date(startTime)).getMonthValue());
        exportDO.setYear(DateUtil.dateToLocalDate(new Date(startTime)).getYear());
        exportDO.setCreateTime(System.currentTimeMillis());
        exportDO.setUpdateTime(System.currentTimeMillis());
        exportDO.setPath(excelProvider.getDir() + filename);
        exportDO.setFileSize(FileUtil.size(new File(excelProvider.getDir() + filename)));
        exportDO.setDimension(dimension.getValue());
        exportDO.setRemark(remark);
        exportDO.setRowCount((long) list.size());
        exportDO.setUseTime((int) (endTimeStart - useTimeStart));

        return excelExportDAO.insert(exportDO) > 0;

    }


    private boolean update(ExportExcel exportExcel, Long startTime, Long endTime, ExportExcelDimensionEnum dimension, String remark) {
        // 计算耗时
        long useTimeStart = System.currentTimeMillis();

        List list = Optional.ofNullable(exportExcel.getRowsList(startTime, endTime)).orElse(new ArrayList());

        String filename = getPath(exportExcel.getFileName(), DateUtil.getIntDate(new Date(startTime)), dimension.getTable());
        // 保存到磁盘
        excelProvider.save2Disk(filename, exportExcel.getSheetName(), list, exportExcel.getClassType());

        long endTimeStart = System.currentTimeMillis();

        ExcelExportDO exportDO = new ExcelExportDO();
        exportDO.setExcelName(exportExcel.getFileName());
        exportDO.setUpdateTime(System.currentTimeMillis());
        exportDO.setFileSize(FileUtil.size(new File(excelProvider.getDir() + filename)));
        exportDO.setRowCount((long) list.size());
        exportDO.setUseTime((int) (endTimeStart - useTimeStart));
        exportDO.setDimension(dimension.getValue());
        exportDO.setRemark(remark);

        UpdateWrapper uw = new UpdateWrapper();
        uw.setPo(exportDO);
        QueryWrapper qw = new QueryWrapper();
        qw.eq("path", excelProvider.getDir() + filename);
        qw.eq("bizType", exportExcel.getBizType().getValue());
        uw.setQw(qw);
        return excelExportDAO.update(uw) > 0;
    }

    private String getPath(String fileName, Integer dateInt, String dimension) {
        return fileName + "-" + dateInt + "-" + dimension + "数据.xlsx";
    }

}
